/* Native Testing Program for the Refefree Programming Contest Judging System
 * Copyright (C) 2005 Wolfgang Hess
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include <iostream>  
#include <iomanip>
#include <string>
#include <sstream>
#include <utility>
#include <cmath>
#include <stdio.h>  
#include <stdlib.h>
#include <sys/types.h> 
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <fcntl.h>  
#include <unistd.h>
#include <dirent.h>


void  parse(char *line, char **argv)
{
     while (*line != '\0') {       /* if not the end of line ....... */ 
          while (*line == ' ' || *line == '\t' || *line == '\n')
               *line++ = '\0';     /* replace white spaces with 0    */
          *argv++ = line;          /* save the argument position     */
          while (*line != '\0' && *line != ' ' && 
                 *line != '\t' && *line != '\n') 
               line++;             /* skip the argument until ...    */
     }
     *argv = '\0';                 /* mark the end of argument list  */
}

void testrun(const char *src, const char *tmp, double timelimit,
	void (*compile)(const char *, const char *,const char *,char *),
	void (*execute)(const char *, const char *),  const char *compilerPath,char *compilerCommand)
{
	// fork the compiler process
	pid_t result = fork();  //pid_t : process ID integer.
	if(result == -1) throw "Fork failed";
	if(result == 0)
	{
		// call the compiler
	//	if(compilerCommand==" ")
	//	std::cerr<<"empty space:"<<compilerCommand<<std::endl;
		compile(src, tmp,compilerPath,compilerCommand);
		std::cerr << "CHILDPROCESS: Compiler did not execute." << std::endl; //insert '/n' and flush.
		exit(1);
	}
	
	int status;
	waitpid(result, &status, 0);
	
	std::cerr << std::endl;
	
	if(!WIFEXITED(status)) throw "Compiler error";
	if(WEXITSTATUS(status) != 0) throw std::make_pair("COMPILE", std::string("Compile error"));
	
	// get the resource for the compilation (before execution)
	struct rusage usage0;
	if(getrusage(RUSAGE_CHILDREN, &usage0) != 0)
		throw "getrusage() failed";
	
	// fork the execution process
	result = fork();
	if(result == -1) throw "Fork failed";
	if(result == 0)
	{
		// change the directory and execute the compiled program
		if(chdir(tmp) != 0)
		{
			std::cerr << "CHILDPROCESS: Unable to chdir into temporary directory." << std::endl;
			exit(1);
		}
		
		// set resource limit
/*		{
			struct rlimit limit;
			limit.rlim_cur = limit.rlim_max = int(std::ceil(timelimit) * 2.0);
			if(setrlimit(RLIMIT_CPU, &limit) != 0)
			{
				std::cerr << "CHILDPROCESS: Unable to limit cpu time with setrlimit()." << std::endl;
				exit(1);
			}
		}
*/		
		execute(src, tmp);
		std::cerr << "CHILDPROCESS: Program did not execute." << std::endl;
		exit(1);
	}

	waitpid(result, &status, 0);
	std::cerr << std::endl;
		
	// get the resource usage and check for timeout
	struct rusage usage;
	if(getrusage(RUSAGE_CHILDREN, &usage) != 0)
		throw "getrusage() failed";
	usage.ru_utime.tv_sec -= usage0.ru_utime.tv_sec;
	usage.ru_utime.tv_usec -= usage0.ru_utime.tv_usec;
	if(usage.ru_utime.tv_usec < 0)
		--usage.ru_utime.tv_sec, usage.ru_utime.tv_usec += 1000000;
	usage.ru_stime.tv_sec -= usage0.ru_stime.tv_sec;
	usage.ru_stime.tv_usec -= usage0.ru_stime.tv_usec;
	if(usage.ru_stime.tv_usec < 0)
		--usage.ru_stime.tv_sec, usage.ru_stime.tv_usec += 1000000;
	
	std::cerr << "Time spent is User=" << usage.ru_utime.tv_sec << "."
		<< std::right << std::setfill('0') << std::setw(6)
		<< usage.ru_utime.tv_usec << "s and Sys=" << usage.ru_stime.tv_sec
		<< "." << std::right << std::setfill('0') << std::setw(6)
		<< usage.ru_stime.tv_usec << "s." << std::endl;
	
	{
		long int sec = usage.ru_utime.tv_sec + usage.ru_stime.tv_sec;
		long int usec = usage.ru_utime.tv_usec + usage.ru_stime.tv_usec;
		double timespent = sec + usec / 1000000.0;
		if(timespent > timelimit)
		{
			std::ostringstream os;
			os << timespent << "s > " << timelimit << "s";
			throw std::make_pair("TIMELIMIT", os.str());
		}
	}
	
	if(!WIFEXITED(status)) throw std::make_pair("RUNTIME", std::string("Program crashed"));
	if(WEXITSTATUS(status) != 0)
	{
		std::ostringstream os;
		os << "Program returned exit code " << WEXITSTATUS(status);
		throw std::make_pair("RUNTIME", os.str());
	}
	
	// check usage of other resources?
	// no!
}

void compile_c(const char *src, const char *tmp,const char *compilerPath, char *compilerCommand)
{
	//parse the output switch ;use substring. the first option is the output switch.
	//std::string outputSwitch=std::string(compilerCommand).
	const std::string target = std::string(tmp) + "/a.out";
	//std::string compilerCommand_(compilerCommand);
	
	//if (std::strcmp(compilerPath,"gcc")==0){
	//	std::cerr<<"gcc"<<"-o"<< target.c_str()<<  "-x"<< "c"<<  "-O2"<< "-Wall"<< "-lm"<< src<<std::endl ;
	//	execlp("gcc", "gcc",  "-x", "c",  "-O2", "-Wall", "-lm", "-o", target.c_str(),src,(char *)NULL);
//	}else{
	
		std::strcat(compilerCommand," ");
		std::strcat(compilerCommand,target.c_str());
		std::strcat(compilerCommand," ");
		std::strcat(compilerCommand,src);
		char  line[1024];
		std::strcpy( line, compilerCommand );
		char *compilerCommand_[64];
		parse(line,compilerCommand_);
		std::cerr<<line<<std::endl;
		if (execvp(*compilerCommand_, compilerCommand_) < 0) {     /* execute the command  */
               printf("*** ERROR: exec failed\n");
               exit(1);
  //      }
	
	}
}

void execute_c(const char *src, const char *tmp)
{
	execl("a.out", "a.out", (char *)NULL);
}

void compile_cc(const char *src, const char *tmp, const char *compilerPath, char *compilerCommand)
{
	const std::string target = std::string(tmp) + "/a.out";

	
//	if (std::strcmp(compilerPath,"g++")==0){
	//split the options into array of char*		std::cerr<<"g++"<<"-o"<< target.c_str()<<  "-x"<< "c++"<<  "-O2"<< "-Wall"<<  src<<std::endl ;
	//	execlp("g++", "g++",  "-x", "c++",  "-O2", "-Wall", "-o", target.c_str(),src,(char *)NULL);
//	}else{
		std::strcat(compilerCommand," ");
		std::strcat(compilerCommand,target.c_str());
		std::strcat(compilerCommand," ");
		std::strcat(compilerCommand,src);
		char  line[1024];
		std::strcpy( line, compilerCommand );
		char *compilerCommand_[64];
		parse(line,compilerCommand_);
		std::cerr<<compilerCommand<<std::endl;
		std::cerr<<line<<std::endl;
		if (execvp(*compilerCommand_, compilerCommand_) < 0) {     /* execute the command  */
               printf("*** ERROR: exec failed\n");
               exit(1);
  //      }
		//execlp(compilerPath, compilerPath,"-x", "c++", "-O2", "-Wall", "-o" , target.c_str(), src,  (char *)NULL);
	//std::cerr<<"compiler options in native are:"<<compilerCommand<<std::endl;
	
//	std::cerr<<compilerPath<<compilerCommand<<compilerCommand<<src<<std::endl;
	//execlp(compilerPath, compilerPath,compilerCommand, target.c_str(), src,  (char *)NULL);
	
	}
}

void execute_cc(const char *src, const char *tmp)
{
	execl("a.out", "a.out", (char *)NULL);
}


void compile_java(const char *src, const char *tmp,const char *compilerPath, char *compilerCommand)
{
	const std::string src2 = std::string(tmp) + "/Solution.java";	
	if(link(src, src2.c_str()) != 0)
	{
		std::cerr << "CHILDPROCESS: Unable to link the source file to the temporary directory." << std::endl;
		exit(1);
	}
	
//if (std::strcmp(compilerPath,"javac")==0){
//	std::cerr<<"javac"<<"-d"<<tmp<<src2.c_str()<<std::endl;
	//execlp("javac", "javac", "-d", tmp, src2.c_str(), (char *)NULL);
//}
//else{
		std::strcat(compilerCommand," ");
		std::strcat(compilerCommand,tmp);
		std::strcat(compilerCommand," ");
		std::strcat(compilerCommand,src2.c_str());
		char  line[1024];
		std::strcpy( line, compilerCommand );
		char *compilerCommand_[64];
		parse(line,compilerCommand_);
		std::cerr<<compilerCommand<<std::endl;
		std::cerr<<line<<std::endl;
		if (execvp(*compilerCommand_, compilerCommand_) < 0) {     /* execute the command  */
               printf("*** ERROR: exec failed\n");
               exit(1);
  //      }
//	std::cerr<<compilerPath<<compilerCommand<<compilerCommand<<src<<std::endl;
	//execlp(compilerPath, compilerPath, compilerCommand,tmp,src2.c_str(), (char *)NULL);
	}
}

void execute_java(const char *src, const char *tmp)
{
	execlp("java", "java", "Solution", (char *)NULL);
}
//here i add the delphi component with borland delphi compiler
void compile_dcc32(const char *src, const char *tmp, const char *compilerPath,char *compilerCommand)
{
//	const std::string src2 = std::string(tmp) +"/d.out";	
    const std::string src2 = std::string(tmp) +"/d.out";
	if(link(src, src2.c_str()) != 0)
	{
		std::cerr << "CHILDPROCESS: Unable to link the source file to the temporary directory." << std::endl;
		exit(1);
	}
  // const std::string src3 = std::string(tmp) +"\\d.out";
//	if (std::strcmp(compilerPath,"dcc32")==0){
	//   std::cerr<<"dcc32"<<"-Q"<<src3.c_str()<<std::endl;
	//   execlp("dcc32", "dcc32","-Q",src2.c_str(),  (char *)NULL);}
	//else{
	  // std::cerr<<compilerPath<<compilerCommand<<compilerCommand<<src<<std::endl;
	  // execlp(compilerPath, compilerPath,compilerCommand,src2.c_str(),  (char *)NULL);
		std::strcat(compilerCommand," ");
		std::strcat(compilerCommand,src2.c_str());
		char  line[1024];
		std::strcpy( line, compilerCommand );
		char *compilerCommand_[64];
		parse(line,compilerCommand_);
		std::cerr<<compilerCommand<<std::endl;
		std::cerr<<line<<std::endl;
		if (execvp(*compilerCommand_, compilerCommand_) < 0) {     /* execute the command  */
               printf("*** ERROR: exec failed\n");
               exit(1);
      //  }
	}
}

void execute_dcc32(const char *src, const char *tmp)
{//	const std::string src2 = std::string(tmp) +"\\d.exe";
//	execlp(src2.c_str(), src2.c_str(), (char *)NULL);
	execlp("d.exe", "d.exe", (char *)NULL);
}

void compile_fpc(const char *src, const char *tmp, const char *compilerPath, char *compilerCommand)
{
//const std::string target = std::string(tmp) + "/d.out";
	
    const std::string src2 = std::string(tmp) +"/d.out";
	if(link(src, src2.c_str()) != 0)
	{
		std::cerr << "CHILDPROCESS: Unable to link the source file to the temporary directory." << std::endl;
		exit(1);
	}

//	if (std::strcmp(compilerPath,"fpc")==0){
   //		std::cerr<<"first branch: "<<"fpc"<<"-v0"<<src2.c_str()<<std::endl;
  // 		execlp("fpc", "fpc", "-v0 ", src2.c_str(),  (char *)NULL);
   	//		execlp("fpc", "fpc", "-v0 ", src,  (char *)NULL);
   
   	//}
 //	else {
 	//	std::cerr<<"second branch:"<<compilerPath<<compilerCommand<<src<<std::endl;
 		//execlp(compilerPath,compilerPath, compilerCommand, src2.c_str(),  (char *)NULL);
	 //   std::strcat(compilerCommand," ");
	//	std::strcat(compilerCommand,target.c_str());
		std::strcat(compilerCommand," ");
		std::strcat(compilerCommand,src2.c_str());
		char  line[1024];
		std::strcpy( line, compilerCommand );
		char *compilerCommand_[64];
		parse(line,compilerCommand_);
		std::cerr<<compilerCommand<<std::endl;
		std::cerr<<line<<std::endl;
		if (execvp(*compilerCommand_, compilerCommand_) < 0) {     /* execute the command  */
               printf("*** ERROR: exec failed\n");
               exit(1);
   //     }
	
	}
}

void execute_fpc(const char *src, const char *tmp)
{
	execlp("d.exe", "d.exe", (char *)NULL);
}


/// this function is (currently) no longer used
// tested until Java 1.4
/*
bool has_java_main(const char *classname)
{
	// fork the javap process
	int fds[2];
	if(pipe(fds) != 0)
	{
		std::cerr << "CHILDPROCESS: Pipe creation failed." << std::endl;
		exit(1);
	}
	pid_t result = fork();
	if(result == -1)
	{
		std::cerr << "CHILDPROCESS: Fork failed." << std::endl;
		exit(1);
	}
	if(result == 0)
	{
		if(dup2(fds[1], STDOUT_FILENO) != STDOUT_FILENO)
		{
			std::cerr << "CHILDPROCESS: Redirecting stdout failed." << std::endl;
			exit(1);			
		}
		execlp("javap", "javap", classname, (char *)NULL);
		std::cerr << "CHILDPROCESS: 'javap' did not execute." << std::endl;
		exit(1);
	}
	if(close(fds[1]) != 0)
	{
		std::cerr << "CHILDPROCESS: Closing pipe failed." << std::endl;
		exit(1);		
	}
	
	std::string javap;
	char ch;
	while(read(fds[0], &ch, 1) == 1)
		javap += ch;
	if(close(fds[0]) != 0)
	{
		std::cerr << "CHILDPROCESS: Closing pipe failed." << std::endl;
		exit(1);		
	}
	int status;
	waitpid(result, &status, 0);
	
	if(!WIFEXITED(status))
	{
		std::cerr << "CHILDPROCESS: 'javap' crashed." << std::endl;
		exit(1);
	}
	if(WEXITSTATUS(status) != 0)
	{
		std::cerr << "CHILDPROCESS: 'javap' did not exit successfully." << std::endl;
		exit(1);
	}
	
	std::istringstream jps(javap);
	std::string line;
	while(std::getline(jps, line))
	{
		if((std::string(line, 0, 47) == "    public static void main(java.lang.String[])") ||
			(std::string(line, 0, 53) == "    public static final void main(java.lang.String[])"))
			return true;
	}
	
	return false;
}
*/

/// this function is (currently) no longer used
// tested until Java 1.4
/*
void execute_java(const char *src, const char *tmp)
{
	DIR *d = opendir(".");
	if(d == NULL)
	{
		std::cerr << "CHILDPROCESS: Unable reading the temporary directory." << std::endl;
		exit(1);
	}
	struct dirent *de;
	bool found = false;
	std::string classname;
	while((de = readdir(d)) != NULL)
	{
		std::string cfile(de->d_name);	// maybe it's the class file
		if(cfile.size() > 6 && std::string(cfile, cfile.size() - 6) == ".class")
		{
			std::string cname(cfile, 0, cfile.size() - 6); 
			// check for the main method
			if(has_java_main(cname.c_str()))
			{
				if(!found)
				{
					classname = cname;
					found = true;
				}
				else
				{
					std::cerr << "CHILDPROCESS: Multiple main methods defined." << std::endl;
					exit(1);					
				}
			}
		}
	}
	if(closedir(d) != 0)
	{
		std::cerr << "CHILDPROCESS: Error closing the temporary directory." << std::endl;
		exit(1);
	}
	
	if(!found)
	{
		std::cerr << "CHILDPROCESS: No main method defined." << std::endl;
		exit(1);					
	}
	
	execlp("java", "java", classname.c_str(), (char *)NULL);
}
*/

/// assumes a main in class Solution


struct temporary
{
	temporary()
	{
		strcpy(temp, "temp.XXXXXX");
		fd = mkstemp(temp);
		if(fd == -1) throw "Unable to create temporary file";
		if(close(fd) == -1) throw "Unable to close temporary file";
	}
	
	~temporary()
	{
		unlink(temp);
	}
	
	char temp[12];
	int fd;
};

struct temporary_dir
{
#ifdef __linux__
	temporary_dir()
	{
		strcpy(temp, "temp.XXXXXX");
		if(mkdtemp(temp) == NULL) throw "Unable to create temporary directory";
	}
#else
	temporary tfile;

	temporary_dir()
	{
		strcpy(temp, tfile.temp);
		temp[3] = 'd';
		if(mkdir(temp, S_IRWXU) == -1) throw "Unable to create temporary directory";
	}
#endif
	
	~temporary_dir()
	{
		if(chdir(temp) != 0)
		{
			std::cerr << "WARNING: Unable to chdir into temporary directory on cleanup." << std::endl;
			return;
		}
		DIR *d = opendir(".");
		if(d == NULL)
		{
			std::cerr << "WARNING: Unable to open temporary directory on cleanup." << std::endl;
			if(chdir("..") != 0)
				throw "Double chdir-fault";
			return;
		}
		struct dirent *de;
		while((de = readdir(d)) != NULL)
		{
			std::string dname(de->d_name);
			if(dname == "." || dname == "..") continue;
			if(unlink(de->d_name) != 0)
				std::cerr << "WARNING: Unable to unlink '" << de->d_name << "'." << std::endl;
		}
		if(closedir(d) != 0)
		{
			std::cerr << "WARNING: Error closing the temporary directory on cleanup." << std::endl;
			if(chdir("..") != 0)
				throw "Double chdir-fault";			
			return;
		}

		if(chdir("..") != 0)
			throw "Double chdir-fault";			
		
		if(rmdir(temp) != 0)
		{
			std::cerr << "WARNING: Unable to remove temporary directory." << std::endl;
			return;
		}
	}
	
	char temp[12];
};


struct lang_info
{
	std::string id;
	std::string compiler;
	void (*compile)(const char*, const char*, const char*,  char*);
	void (*execute)(const char*, const char*);

} languages[] = {
	{ "c", "gcc",compile_c, execute_c },
	{ "cc", "g++",compile_cc, execute_cc },
	{ "java", "javac",compile_java, execute_java },
	{"pas","fpc",compile_fpc, execute_fpc},
	{"pas","dcc32",compile_dcc32, execute_dcc32}


};

template<typename T, unsigned N> unsigned array_size(T(&)[N]) { return N; }


int main(int argc, char *argv[])
{
	try
	{  
		if(argc < 10) throw "Too few arguments";
		
		{
			
			//open the .err file as standard err output stream 
			int fd;
			fd = open(argv[6], O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
			if(fd == -1) return 2;
			if(dup2(fd, STDERR_FILENO) == -1) return 2;

			//open the test input file for standard input reading
			fd = open(argv[4], O_RDONLY);
			if(fd == -1) throw "Critical error (stdin)";
			if(dup2(fd, STDIN_FILENO) == -1) throw "Critical error (stdin-dup)";
			
			//open the .out file for output
			fd = open(argv[5], O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
			if(fd == -1) throw "Critical error (stdout)";
			if(dup2(fd, STDOUT_FILENO) == -1) throw "Critical error (stdout-dup)";
		}
		temporary_dir tmp;
		
		const std::string language_id(argv[1]);
		const std::string compiler(argv[7]);
		const char *compilerPath=argv[8];
		//concaten argv[9] onwards

		char *compilerCommand=argv[8];
		int i=0;
	//	std::cerr<<9<<":"<<argv[9]<<std::endl;
		
		while (i+9<argc){
			i=i+1;
			std::strcat(compilerCommand," ");
	//		std::cerr<<i+9<<":"<<argv[9+i]<<std::endl;
			std::strcat(compilerCommand,argv[8+i]);
		}
		double timelimit = 0.0;
		{
			std::istringstream is(argv[2]);
			if(!(is >> timelimit) || timelimit < 0.0 || timelimit > 3600.0)
				throw "Timelimit must be 0.0 - 3600.0 seconds";
		}
				
		if(std::strcmp(language_id.c_str(),"pas")==0&&std::strcmp(compiler.c_str(),"dcc32")==0)
		{
			testrun(argv[3], tmp.temp, timelimit, languages[5].compile, languages[5].execute, compilerPath,compilerCommand);
			goto found;
		}
			
		
		for(unsigned i=0; i<array_size(languages); ++i)
		{
			if(languages[i].id == language_id)
		//	if(languages[i].id == language_id&&languages[i].compiler==compiler)
			{
				testrun(argv[3], tmp.temp, timelimit, languages[i].compile, languages[i].execute, compilerPath,compilerCommand);
				goto found;
			}
			
		}
		throw "Unsupported Language";
		found:;
		
	}
	catch(const char *e)
	{
		std::cerr << "ERROR: " << e << "." << std::endl;
		return 1;
	}
	catch(const std::pair<const char *, std::string> &e)
	{
		std::cerr << e.first << ": " << e.second << "." << std::endl;
		return 0;
	}
	catch(...)
	{
		std::cerr << "ERROR: Uncaught exception." << std::endl;
		return 1;
	}
	
	std::cerr << "SUCCESS: Success." << std::endl;
}
