/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 27.04.2005
 *
 */
package de.tudarmstadt.yajs;


import java.io.IOException;
import java.io.Reader;
import java.net.InetSocketAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Iterator;
import java.util.List;

import javax.swing.JOptionPane;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;

import de.tudarmstadt.lang.Lang;
import de.tudarmstadt.yajs.lib.CCompiler;
import de.tudarmstadt.yajs.lib.ContestInfo;
import de.tudarmstadt.yajs.lib.Language;
import de.tudarmstadt.yajs.lib.LanguageSet;
import de.tudarmstadt.yajs.lib.Problem;
import de.tudarmstadt.yajs.lib.ProblemSet;

/** Handles reading from (and maybe --- at some point --- writing to) configuration files.
 * @author markus
 */
/**
 * @author O, Hung Lun
 * @date 11.01.2007
 */
public class ConfigurationIO {
	
	/** Read a configuration file.
	 * @param reader The source to read from.
	 * @return A ContestInfo object containing created from the configuration file.
	 * @throws IOException
	 */
	public ContestInfo readConfiguration(Reader reader) throws ConfigurationException {
		try {
			org.jdom.Document document = parseDocument(reader);
			InetSocketAddress server = getServer(document);
			ProblemSet problems = getProblems(document);
			LanguageSet languages = getLanguages(document);
			URI testBase = getTestBase(document);
			String guiLanguage = getGUILanguage(document);
			String mode=getMode(document);
			return new ContestInfo(server, languages, problems, testBase, guiLanguage,mode);
		} catch (JDOMException e) {
			throw new ConfigurationException(e.getMessage());
		} catch (IOException e) {
			throw new ConfigurationException(e.getMessage());
		}
	}

	/** Extract the test base URI. This URI indicates where the test data
	 * hierarchy is stored. 
	 * @param document An org.jdom.Document.
	 * @return Returns the URI under which the test data is stored.
	 * @throws JDOMException
	 * @throws ConfigurationException
	 */
	private URI getTestBase(Document document) throws JDOMException, ConfigurationException {
		try {
			Element e = (Element) XPath.selectSingleNode(document, "/client-configuration/test-base"); //$NON-NLS-1$
			if(e == null) return null; // MG throw new ConfigurationException("Test-base must be set.");
			String testbase = e.getTextTrim();
			if(!testbase.endsWith("/")) testbase = testbase + "/"; //$NON-NLS-1$ //$NON-NLS-2$
			URI tb = new URI(testbase);
			if(!tb.isAbsolute() || tb.isOpaque()) throw new ConfigurationException("Non-hierarchical or opaque URI given. Test-base must be a hierarchical absolute URI (e.g. http://www.tests.net/sampledata/)");  //$NON-NLS-1$
			if(!tb.getScheme().equals("http") && !tb.getScheme().equals("file")) throw new ConfigurationException("Error reading test-base: Only file and http schemes are supported.");  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			return tb;
		} catch (URISyntaxException e1) {
			throw new ConfigurationException(Lang.getString("ConfigurationIO.11")); //$NON-NLS-1$
		}		
	}
	
	/**Gets the GUI language from configuration file.
	 * @param Document   configuration document
	 * @return      String the language for GUI
	 * @see         
	 */
	public String getGUILanguage(Document document) throws JDOMException, ConfigurationException {
		try {
			Element e = (Element) XPath.selectSingleNode(document, "/client-configuration/GUILanguage"); //$NON-NLS-1$
			if(e == null) throw new ConfigurationException("GUI Language must be set in configuration file."); //$NON-NLS-1$
			String guilanguage = e.getTextTrim();
			return 	guilanguage;
		} catch (JDOMException e) {
			throw new ConfigurationException(e.getMessage());
		} 	
	}
	/**Gets the mode from configuration file.
	 * @param Document   configuration document
	 * @return      String the mode for Refefree
	 * @see         
	 */
	public String getMode(Document document) throws JDOMException, ConfigurationException {
		try {
			Element e = (Element) XPath.selectSingleNode(document, "/client-configuration/mode"); //$NON-NLS-1$
			if(e == null) return "online"; //$NON-NLS-1$
			String mode = e.getTextTrim();
			return 	mode;
		} catch (JDOMException e) {
			throw new ConfigurationException(e.getMessage());
		} 	
	}
	

	/** Extract all languages from the given org.jdom.Document.
	 * @param document An org.jdom.Document.
	 * @return A LanguageSet object containing all languages listed in document.
	 * @throws JDOMException
	 */
	private static LanguageSet getLanguages(Document document) throws JDOMException ,ConfigurationException{
		LanguageSet languages = new LanguageSet();
		List languageNodes = XPath.selectNodes(document, "/client-configuration/language"); //$NON-NLS-1$
		Iterator i = languageNodes.iterator();
		while(i.hasNext())
		{
			Object thisLanguage = i.next();
			Element id = (Element) XPath.selectSingleNode(thisLanguage, "id"); //$NON-NLS-1$
			if(id == null) {
			 JOptionPane.showMessageDialog(null, Lang.getString("ConfigurationIO.12")); //$NON-NLS-1$
			 throw new ConfigurationException(Lang.getString("ConfigurationIO.13")); //$NON-NLS-1$
			}
			Element name = (Element) XPath.selectSingleNode(thisLanguage, "name"); //$NON-NLS-1$
			if(name == null) {JOptionPane.showMessageDialog(null, Lang.getString("ConfigurationIO.15")); //$NON-NLS-1$
			throw new ConfigurationException(Lang.getString("ConfigurationIO.16")); //$NON-NLS-1$
			}
			Element compilerName=(Element) XPath.selectSingleNode(thisLanguage, "compiler/name"); //$NON-NLS-1$
			if(compilerName == null){ JOptionPane.showMessageDialog(null, Lang.getString("ConfigurationIO.19")); //$NON-NLS-1$ //$NON-NLS-2$
			throw new ConfigurationException(Lang.getString("ConfigurationIO.20")); //$NON-NLS-1$
			}
			Element compilerOption=(Element) XPath.selectSingleNode(thisLanguage, "compiler/option"); //$NON-NLS-1$
			if(compilerOption == null){ JOptionPane.showMessageDialog(null, Lang.getString("ConfigurationIO.19")); //$NON-NLS-1$ //$NON-NLS-2$
			throw new ConfigurationException(Lang.getString("ConfigurationIO.20")); //$NON-NLS-1$
			}
			//TODO error message for compiler 
			Element compilerPath=(Element) XPath.selectSingleNode(thisLanguage, "compiler/path"); //$NON-NLS-1$
			if(compilerPath == null){ JOptionPane.showMessageDialog(null, Lang.getString("ConfigurationIO.19")); //$NON-NLS-1$ //$NON-NLS-2$
			throw new ConfigurationException(Lang.getString("ConfigurationIO.20")); //$NON-NLS-1$
			}
			Element editor=(Element) XPath.selectSingleNode(thisLanguage, "editor"); //$NON-NLS-1$
			if(editor == null) {JOptionPane.showMessageDialog(null, Lang.getString("ConfigurationIO.22")); //$NON-NLS-1$
			throw new ConfigurationException(Lang.getString("ConfigurationIO.23")); //$NON-NLS-1$
			}
			
			CCompiler compiler=new CCompiler(compilerName.getTextTrim(),compilerPath.getTextTrim(),compilerOption.getTextTrim());
			languages.add(new Language(id.getTextTrim(),name.getTextTrim(),compiler,editor.getTextTrim()));
		}
		return languages;
	}

	/** Extract all problems from the given org.jdom.Document.
	 * @param document An org.jdom.Document.
	 * @return A ProblemSet object containing all problems listed in document.
	 * @throws JDOMException
	 */
	private static ProblemSet getProblems(Document document) throws JDOMException {
		ProblemSet problems = new ProblemSet();
		List languageNodes = XPath.selectNodes(document, "/client-configuration/problem"); //$NON-NLS-1$
		Iterator i = languageNodes.iterator();
		while(i.hasNext())
		{
			Object thisProblem = i.next();
			Element id = (Element) XPath.selectSingleNode(thisProblem, "id"); //$NON-NLS-1$
			Element name = (Element) XPath.selectSingleNode(thisProblem, "name"); //$NON-NLS-1$
			Element tlimit = (Element) XPath.selectSingleNode(thisProblem, "time-limit");
		
			try{
			  problems.add(new Problem(id.getTextTrim(), name.getTextTrim(),Double.parseDouble(tlimit.getTextTrim())));
			}
			catch(NumberFormatException e){
				JOptionPane.showMessageDialog(null, Lang.getString("timelimitexception")+" "+name.getTextTrim()+"\n"+
						Lang.getString("timelimitexception2")+" "+name.getTextTrim());
			}
			finally{
				 problems.add(new Problem(id.getTextTrim(), name.getTextTrim(),10));
					
				
			}
		}
		return problems;
	}

	/** Extract the server information from the given org.jdom.Document.
	 * @param document An org.jdom.Document.
	 * @return The server information contained in the document.
	 * @throws JDOMException
	 */
	private static InetSocketAddress getServer(Document document) throws JDOMException {
		Object server = XPath.selectSingleNode(document, "/client-configuration/server"); //$NON-NLS-1$
		Element host = (Element) XPath.selectSingleNode(server, "hostname"); //$NON-NLS-1$
		Element port = (Element) XPath.selectSingleNode(server, "port");  //$NON-NLS-1$
		return new InetSocketAddress(host.getTextTrim(), Integer.valueOf(port.getTextTrim()).intValue());
	}

	/** Read the document from a Reader (most probably a FileReader) and build an
	 * org.jdom.Document. 
	 * @param reader The Reader to read from.
	 * @return An org.jdom.Document representing the XML document that was read.
	 * @throws JDOMException
	 * @throws IOException
	 */
	private Document parseDocument(Reader reader) throws JDOMException, IOException {
		org.jdom.input.SAXBuilder builder = new org.jdom.input.SAXBuilder();
		return builder.build(reader);
	}
}
