/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 15.03.2005
 *
 */
package de.tudarmstadt.yajs;

import de.tudarmstadt.lang.*;
import de.tudarmstadt.yajs.gui.LoginDialog;
import de.tudarmstadt.yajs.gui.NewClarificationRequestDialog;
import de.tudarmstadt.yajs.gui.contestant.events.EditorEvent;
import de.tudarmstadt.yajs.lib.ClarificationRequest;
import de.tudarmstadt.yajs.lib.Client;
import de.tudarmstadt.yajs.lib.ClientClarificationAnswer;
import de.tudarmstadt.yajs.lib.ConnectionError;
import de.tudarmstadt.yajs.lib.ContestInfo;
import de.tudarmstadt.yajs.lib.Language;
import de.tudarmstadt.yajs.lib.LanguageSet;
import de.tudarmstadt.yajs.lib.Login;
import de.tudarmstadt.yajs.lib.Problem;
import de.tudarmstadt.yajs.lib.ProblemSet;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.Writer;
import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Iterator;
import java.util.Timer;
import java.util.TimerTask;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
/**
 * Base class for other client applications. Defines some basic functionality  (every client app has a client object, a GUI etc...)
 * @author  markus
 */
/**
 * @author O, Hung Lun
 * @date 11.01.2007
 */
public abstract class YAJSClientApp {
	
	/** The maximum file size to read (10M)
	 */
	protected static final int MAX_FILESIZE = 1024 * 1024 * 1024; 
	
	/**
	 * The client object.
	 * Maybe misnomer, because it mainly serves as a server-stub. 
	 */
	protected Client client;
	/**
	 * The login. Name and password.
	 */
	private Login login;
	/**
	 * Information about the contest (server address, problems, languages). 
	 */
	protected ContestInfo contestinfo;
	/**
	 * A timeout before which the login has to be completed.
	 * (in ms)
	 */
	private static final long LOGIN_TIMEOUT = 5000;
	/**
	 * A TimerTask for the above timeout.
	 * Needs to be stored, because we want to cancel
	 * it in case the login is completed.
	 */
	protected TimerTask logintimeout;
	
	/** Boolean flag indicating whether the login was successful. */
	protected boolean loggedIn = false;

	/** Constructor that takes a ContestInfo object.
	 * @param contestinfo Information about the contest.
	 */
	public YAJSClientApp(String clientConfigFilename) {
		try {
			contestinfo=readContestInfo(clientConfigFilename);
		} catch (ConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Lang.setLanguage(contestinfo.getGuiLanguage());
		this.logintimeout = new TimerTask(){
			public void run() {
				close();
				JOptionPane.showMessageDialog(null, Lang.getString("Theattempttologintimedout.Thisisprobablyduetoaproblemwiththeserverorthenetwork"), Lang.getString("Error"), JOptionPane.ERROR_MESSAGE);
			}
		};
	}


	/** Cancel the login timeout.
	 * Needed when the login is completed.
	 */
	protected void cancelLoginTimeout()
	{
		if(this.logintimeout != null)
		{
			logintimeout.cancel();
		}
		loggedIn = true;
	}
	/** Abstract function. Subclasses should overwrite this method to return the name of the program. */
	protected abstract String getProgramName();
	
	/**
	 * Get the login. TODO This definitely needs refactoring.
	 * @return
	 * @uml.property  name="login"
	 */
	protected Login getLoginGUI(final String usertype) {
		if(login == null)
		{
			try {
				javax.swing.SwingUtilities.invokeAndWait(new Runnable() {
				    @SuppressWarnings("deprecation")
					public void run(){
				    	LoginDialog dlg = new LoginDialog(getGUI(), getProgramName(),usertype);
				    	dlg.setLocationRelativeTo(null);
				    	dlg.addWindowListener(new WindowAdapter() {
				    		/* (non-Javadoc)
							 * @see java.awt.event.WindowAdapter#windowClosing(java.awt.event.WindowEvent)
							 */
							public void windowClosing(WindowEvent arg0) {
								super.windowClosing(arg0);
								System.exit(0);
							}
				    	});
				    	
						dlg.show();
						login = dlg.getLogin();
				    }
				});
			} catch (InterruptedException e) {
				e.printStackTrace();
				close();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
				close();
			}
		}
		return login;
	}

	/** The run method.
	 * This thread does nothing but process incoming messages.
	 * @return The program will return 0 on success, and a value < 0 on error.
	 */
	public int run() {
		client.reset();
		new Timer().schedule(logintimeout, LOGIN_TIMEOUT);
		
		try {
			while(client.isAlive())
			{
				client.process();
			
				Thread.sleep(25);
			}
			return 0;
		} 
		catch (ConnectionError e) {
			showErrorMessage(e.getMessage());
			return -2;
		} 
		catch (Exception e) {
			e.printStackTrace();
			return -1;
		} finally {
			cancelLoginTimeout();
			close();
			dispose();	
		}
	}
	
	/**	The run method used during offline mode.
	 * It keeps send the signal, login with success.   
	 * The thread keeps running, until the client GUI is closed.   
	 */
	public void runOffline() {
		client.reset();
		client.signalLogin(true);
		
		try {
			while(!OfflineHelper.clienGUIIsClosed())
			{	
				Thread.sleep(1025);;
			}
		} 
		catch (Exception e) {
			e.printStackTrace();
		
		} finally {
			close();
			dispose();	
		}
	}
	/** The client is being closed. Please close all windows...
	 */
	protected void dispose() {}

	/** Close the connection.
	 */
	public void close() {
		if(client != null)
		{
			client.closeSocket();
		}
	}

	/** Show an error message to the user. */
	protected abstract void showErrorMessage(String message);
	
	/** Reads a configuration file.
	 * @param filename The filename of the configuration file.
	 * @return A ContestInfo object created from the configuration file.
	 * @throws IOException
	 * @throws FileNotFoundException
	 */
	public static ContestInfo readContestInfo(String filename) throws ConfigurationException {
		try {
			return new ConfigurationIO().readConfiguration(new FileReader(filename));
		} catch (FileNotFoundException e) {
			throw new ConfigurationException(e.getMessage());
		} 
	}
	
	/** Starts the heartbeat signal.
	 * Periodically sends a heartbeat request.
	 * Useful for receiving server status, if available.
	 */
	protected void startHeartbeatSignal() {
		TimerTask heartbeat = new TimerTask() {
			public void run() {
				client.sendHeartbeatRequest();
			}
		};
		new Timer().scheduleAtFixedRate(heartbeat, 500, 1500);
	}
	
	/** Setter for the LanguageSet. 
	 */
	protected void setLanguageSet(LanguageSet languages) {
		contestinfo.setLanguages(languages);
	}
	
	/** Setter for the ProblemSet.
	 */
	protected void setProblemSet(ProblemSet problems) {
		contestinfo.setProblems(problems);
	}
	
	/** Getter for the ProblemSet.
	 * @return Returns contestinfo.getProblems().
	 */
	protected ProblemSet getProblemSet() {
		return contestinfo.getProblems();
	}
	
	/** Getter for the LanguageSet.
	 * @return Returns contestinfo.getLanguages().
	 */
	protected LanguageSet getLanguageSet() {
		return contestinfo.getLanguages();
	}
	
	/** Getter for the whole contestinfo-object.
	 * Needed to register PropertyChangeListeners.
	 * @return
	 */
	protected ContestInfo getContestInfo() {
		return contestinfo;
	}
	
	/** Get the top-level gui window.
	 * Needed for various dialogs.
	 * @return Returns the JFrame that is the top-level gui window.
	 */
	protected abstract JFrame getGUI();
	
	/** Read everything from a Reader.
	 * @param reader The Reader to read from. 
	 * @return Returns a String containing all characters read.
	 * @throws IOException
	 */
	protected String readReaderContents(Reader reader) throws IOException {
		StringBuffer retval = new StringBuffer();
		char[] buf = new char[1024];
		int len;
		while((len = reader.read(buf)) >= 0)
		{
			retval.append(buf, 0, len);
			if(retval.length() > MAX_FILESIZE) throw new IOException(Lang.getString("Filetoolarge"));
		}
		return retval.toString();
	}
	
	/** Read from some URI.
	 * Depending on the URI scheme, this reads from files or the web.
	 * @param uri The URI to read from.
	 * @return Returns a String containing the contents located under uri.
	 * @throws IOException
	 */
	protected String readURIContents(URI uri) throws IOException {
		return readReaderContents( getURIReader(uri) );
	}
	
	/** Read from a file.
	 * This method reads and returns the whole content of a file at once.
	 * @param file The file to read from
	 * @return Returns a String containing the contents of the file.
	 * @throws IOException 
	 */
	protected String readFileContents(File file) throws IOException {
		return readReaderContents( getFileReader(file) );
	}
	
	/** Get a Reader for URLs.
	 * @param url The URL to read from.
	 * @return Returns a reader connected to this URL's data.
	 * @throws IOException
	 */
	protected Reader getURLReader(URL url) throws IOException {
		return new InputStreamReader(url.openStream());
	}
	
	/** Get a Reader for Files.
	 * @param file The file to read from.
	 * @return Returns a FileReader for file.
	 * @throws IOException
	 */
	protected Reader getFileReader(File file) throws IOException {
		return new FileReader(file);
	}
	
	/** Get a suitable Reader for the given URI.
	 * Depending on the URI's scheme, this method chooses a suitable reader.
	 * @param uri A URI object.
	 * @return Returns a Reader suitable to read data from uri.
	 * @throws IOException
	 */
	protected Reader getURIReader(URI uri) throws IOException {
		String scheme = uri.getScheme();
		Reader r=getFileReader(new File(uri));
		if(scheme.equals("http")) return getURLReader(uri.toURL());
		else if(scheme.equals("file")) return getFileReader(new File(uri));
		else throw new IOException(Lang.getString("UnknownURIschemeingetURIReader") + scheme);

	}
	
	/** Tests whether the directory containing the test data for problem exists.
	 * @param problem A problem.
	 * @return Returns true iff the directory containing the test data for problem exists.
	 * @throws IOException
	 */
	protected boolean checkTestDirectoryExists(Problem problem) throws IOException {
		 return getContestInfo().getLocalTestLocation(problem).exists();
	}
	
	/** Fetch remotely stored test data and store locally.
	 * Calls <code>fetchRemoteTestData(problem, true)</code>
	 * @param problem The problem to fetch test data for.
	 * @throws IOException
	 */
	protected void fetchRemoteTestData(Problem problem) throws IOException {
		String [] sin={"sin"};
		String[] sout={"sout"};
		boolean carryOn=fetchRemoteTestData(problem, null,sin,true);
		if(carryOn)fetchRemoteTestData(problem, null,sout,false);
		
	}
	
	/** Fetch remotely stored test data and store locally.
	 * If the <code>askOverwrite</code> 
	 * @param problem The problem to fetch test data for.
	 * @param askOverwrite True iff we should ask the user before overwriting files.
	 * @throws IOException
	 */
	
//	protected void fetchRemoteTestData(Problem problem,boolean askOverwrite) throws IOException {
//		File localInput = getContestInfo().getLocalTestInputLocation(problem);
//		File localOutput = getContestInfo().getLocalTestOutputLocation(problem);		
//		
//		try {
//			copyFromURI(getContestInfo().getRemoteTestInputLocation(problem), getContestInfo().getLocalTestInputLocation(problem), askOverwrite);
//		} catch(IOException e) {
//			throw new IOException(Lang.getString("Couldnotfetchtestdatafrom") + getContestInfo().getRemoteTestInputLocation(problem) + "\n" + e.getMessage());
//		}
//		try {
//			copyFromURI(getContestInfo().getRemoteTestOutputLocation(problem), getContestInfo().getLocalTestOutputLocation(problem), askOverwrite);
//		} catch(IOException e) {
//			throw new IOException(Lang.getString("Couldnotfetchtestdatafrom") + getContestInfo().getRemoteTestOutputLocation(problem) + "\n" + e.getMessage());
//		}	
//		
//	}
	
	/** Fetches sample data for all problems, unless a corresponding
	 *  directory already exists. Calls <code>fetchRemoteTestData(problem)</code> for every problem <code>problem</code>.
	 */
	protected void fetchAllRemoteTestData() throws IOException {
		Iterator iter = getContestInfo().getProblems().iterator();
		while(iter.hasNext())
		{
			Problem problem = (Problem) iter.next();
//			String zipFilename="a";
//			try {
//				zipFilename = getContestInfo().getRemoteTestZipLocation(problem).toString();
//			} catch (URISyntaxException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
//			System.err.println(zipFilename);
//			
			if(!checkTestDirectoryExists(problem)){
				fetchRemoteTestData(problem);
			}
		}
	}
	public boolean fetchRemoteTestData (Problem problem, Language language, String[] offlineData,boolean askOverwrite)throws IOException{
		String zipFilename="";
		URI	tmp=null;
		boolean isHttp=false;
		try {
			tmp = getContestInfo().getRemoteTestZipLocation(problem);
			if (tmp.getScheme().equals("http")) isHttp=true;
			
			if (!isHttp){zipFilename=new File(tmp).getCanonicalPath();}
			//ZipFile b= new ZipFile("http://www.3fonline.de/private/problemSet/Stamps.zip");	
		} catch (URISyntaxException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} //$NON-NLS-1$ //$NON-NLS-2$
		
		String problemDir=System.getProperty("user.dir")+File.separator+problem.getName(); //$NON-NLS-1$
		
		boolean copyFromZipSuccessful=true;
		//	String templateFilename=System.getProperty("user.dir")+File.separator+problem.getName()+
//				File.separator+	problem.getID()+"."+language.getID(); //$NON-NLS-1$ //$NON-NLS-2$
		for (int j=0;j<offlineData.length;j++){
			int confirmDialogResult=JOptionPane.NO_OPTION;
			boolean zipEntryExists=false;
			String fileInProblemDir=problemDir+File.separator+offlineData[j];
			String missingFiles="";
			try {	
//				int zipSize=0;
//				if(!isHttp){ZipFile a= new ZipFile(zipFilename);
//				zipSize=a.size();}	
//				
//				
//				if(isHttp)zipSize=10;
				ZipInputStream in =null;
				
				if (!isHttp) {in= new ZipInputStream(new FileInputStream(zipFilename));}
				if (isHttp) {in =new ZipInputStream(tmp.toURL().openStream());}
				String zipEntryName;
		//      check if the template exists in zip file.
		        while(in.available()==1){    	
			    	zipEntryName=in.getNextEntry().getName();		     	
			    	if (zipEntryName.equals(offlineData[j])){ //$NON-NLS-1$ //$NON-NLS-2$
			    		zipEntryExists= true;   
			    		if (zipEntryExists)break;
			   	   	}
			    }
		    	//if no, throw a dialog showing template missing and return false.	
		        if (!zipEntryExists){
		        	missingFiles=missingFiles+offlineData[j];
		        	copyFromZipSuccessful=false;
		        	
		        }	
		   
				if (zipEntryExists){
					FileOutputStream outputFile;
		//			if yes, check if the file already exists in problem directory
					if((new File(fileInProblemDir)).exists()){
		//				if file exists, ask for confirmation.		
						if(offlineData[j].equalsIgnoreCase("sin"))
							offlineData[j]="sin, sout";
						 if(askOverwrite)confirmDialogResult=JOptionPane.showConfirmDialog
						  	(null, Lang.getString("YAJSContestantApp.57")+" "+problem.getName()+File.separator+offlineData[j],"",JOptionPane.YES_NO_OPTION);
		//			if confirmation return true, then copy and open.
						 if (!askOverwrite||confirmDialogResult==JOptionPane.YES_OPTION){
							 if (!(new File(problemDir)).exists() ){
							 		(new File(problemDir)).mkdirs();								    		
								}	
							 outputFile=new FileOutputStream(fileInProblemDir);
							 overWriteFromZipFile(outputFile,in);
							 break;
						 }
							//if confirmation return false, then simply return false.
						 else 	copyFromZipSuccessful=false;
					        
					}
		//			if file does not exist, copy and open the template silently.
			  		else {
						if (!(new File(problemDir)).exists() ){
						 		(new File(problemDir)).mkdirs();								    		
							}	
						 	outputFile=new FileOutputStream(fileInProblemDir);	
						 	overWriteFromZipFile(outputFile,in);
						 	
					}
				}
			in.close();}
			catch (IOException e) {	
				showErrorMessage(Lang.getString("Couldnotfetchtestdatafrom") +zipFilename+"\n" + e.getMessage()); //$NON-NLS-1$ //$NON-NLS-2$
			    return false;
			}	
			if (missingFiles!="")showErrorMessage(missingFiles+" missing in zip file");
		}
		return copyFromZipSuccessful;
	}

	protected void overWriteFromZipFile(FileOutputStream out, ZipInputStream in){
      	     	
      	byte[] buf = new byte[1024];
		        int len;
		        try {
					while ((len = in.read(buf)) > 0) {
					    out.write(buf, 0, len);
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
		      
      }
	/** Get the timelimit for a problem. 
	 * This is later given to the <code>Tester</code>.
	 */
	protected double getTimeLimit(Problem problem)  {
		try {	
//			if( isOnline())
//				return Double.parseDouble(readURIContents(getContestInfo().getRemoteTestLocation(problem).resolve("tlimit")));
//			else
////read from configuration file
				return contestinfo.getProblems().getProblem(problem.getID()).getTlimit();
				
//				return Double.parseDouble(readReaderContents(new FileReader(new
//					File(System.getProperty("user.dir")+File.separator+
//					problem.getName()+File.separator+"tlimit"))));
		}
		 catch (NumberFormatException e) {
			JOptionPane.showMessageDialog(null, e.getMessage() +Lang.getString("Timelimitforproblem") + problem + Lang.getString("giveninwrongformat.")+"\n"+Lang.getString("Pleasecontactstaff")+Lang.getString("YAJSContestantApp.35"), Lang.getString("Error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
		} 
		return 1000;
	}

	/** Private inner interface extending Runnable in order to be able to acquire a return value. */
	public interface MyRunnable extends Runnable {

		public int getReturnValue();
	}
	
	/** Copies the contents from a URI to a local file.
	 * @param remoteTestInputLocation The remote location, i.e. the source.
	 * @param localTestInputLocation The local location, i.e. the destination.
	 * @param askOverwrite True iff we should ask the user before overwriting files.
	 * @throws IOException
	 */
	private void copyFromURI(final URI remoteLocation, final File localLocation, boolean askOverwrite) throws IOException {
		if(localLocation.exists()) {
			if(!localLocation.isFile()) {
				throw new IOException(Lang.getString("Destination") + localLocation + Lang.getString("existsbutisnotaregularfile"));
			}
			if(askOverwrite) {
				MyRunnable confirmOverwrite = new MyRunnable() {
					int returnValue = JOptionPane.NO_OPTION;
					public void run() {
						returnValue = JOptionPane.showConfirmDialog(getGUI(), Lang.getString("File") + localLocation + Lang.getString("existsdoyoureallywanttooverwriteit"), Lang.getString("Fileexists"), JOptionPane.YES_NO_OPTION);
					}
					public int getReturnValue() {
						return returnValue;
					}
				};
				try {
					SwingUtilities.invokeAndWait(confirmOverwrite);
				} catch (Exception e) {
					throw new IOException(e.getMessage());
				}
				if(confirmOverwrite.getReturnValue() != JOptionPane.YES_OPTION)
					return;
			}
		}
	
		
		Reader inputData = getURIReader(remoteLocation);
		File directory = localLocation.getParentFile();
		if(directory != null) directory.mkdirs();
		Writer fw = new FileWriter(localLocation);
		try {
			char[] buffer = new char[1024];
			int len;
			while((len = inputData.read(buffer, 0, buffer.length)) >= 0)
			{
				fw.write(buffer, 0, len);
			}
		} finally {
			inputData.close();
			fw.close();
		}
	}

	/** Pop up a clarification dialog.
	 */
	protected void showNewClarificationDialog() {
		final NewClarificationRequestDialog dlg = new NewClarificationRequestDialog(getGUI(), getProblemSet());
		dlg.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				getClient().requestClarification(new ClarificationRequest(dlg.getSelectedProblem(), dlg.getText()));
			}
		});
		dlg.setLocationRelativeTo(getGUI());
		dlg.show();
	}
	
	/**
	 * Getter for client.
	 * @return  Returns the client.
	 * @uml.property  name="client"
	 */
	protected Client getClient() {
		return client;
	}

	/** Show a clarification in a dialog.
	 * @param answer
	 */
	protected void showClarificationAnswer(final ClientClarificationAnswer answer) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				String title = (answer.getProblem() == null) 
					? Lang.getString("Answertogeneralquestion")
					: Lang.getString("Clarificationregardingproblem") + answer.getProblem().getID();
								
				JOptionPane.showMessageDialog(getGUI(), answer.getAnswer(), title, JOptionPane.INFORMATION_MESSAGE);
			}
		});
	}
	
	/**
	 * Getter for loggedIn
	 * @return  Returns whether the client is already logged in.
	 * @uml.property  name="loggedIn"
	 */
	protected boolean isLoggedIn() {
		return loggedIn;
	}
	
	/** Show a dialog that asks for a programming language */
	protected Language askForLanguage() {
		Object[] selections = getContestInfo().getLanguages().toArray();
		Object result=JOptionPane.showInputDialog(getGUI(),Lang.getString("PleaseSpecifiyALanguage"), Lang.getString("Selectalanguage"), JOptionPane.QUESTION_MESSAGE, null, selections, selections[0]);
		if (result!=null) return (Language)result;
		return null;
		//if ok is clicked, then return language else return null;
	}	


	public boolean isOnline() {
		// TODO Auto-generated method stub
		return login.isOnline();
	}
	protected void setLogin(Login l){
		login=l;
	}


	



	

}
