/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * 
 * Created on 24.05.2005
 *
 */
package de.tudarmstadt.yajs.gui;

import de.tudarmstadt.events.ActionEventGenerator;
import de.tudarmstadt.lang.*;
import de.tudarmstadt.yajs.lib.Problem;
import de.tudarmstadt.yajs.lib.ProblemSet;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
/**
 * A dialog that allows writing a clarification request.
 * @author  markus
 */
public class NewClarificationRequestDialog extends JDialog {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -5392707731887732017L;

	/**
	 * A wrapper around a problem. This allows homogenously selecting  whether the question is a general one, or a question regarding a problem.
	 */
	private static class ProblemWrapper {
		/** The encapsulated problem. <code>null</code> if this represents the "General" question. */
		private Problem problem;
		/** Constructor. Wraps a problem. 
		 * @param problem The problem around which to wrap, <code>null</code> if this is meant to be a general question. */
		public ProblemWrapper(Problem problem) {
			this.problem = problem;
		}
		/**
		 * Getter for the problem.
		 * @uml.property  name="problem"
		 */
		public Problem getProblem() {
			return problem;
		}
		
		/** Get a string representation. In case of a general question, this is "General question", otherwise a string representation of the problem. */
		public String toString() {
			if(problem == null) return Lang.getString("Generalquestion");
			return problem.getID() + " - " + problem.getName();
		}
	}
	
	private ActionEventGenerator actionSignal;
	private JComboBox problemSelection;
	private JEditorPane textField;
	private JButton okButton;
	private JButton cancelButton;
	
	private ProblemSet problems;
	
	/** Constructor. Create a clarification request dialog with a JFrame as parent and problems from the given set.
	 * @param parent The parent frame.
	 * @param problems The ProblemSet.
	 */
	public NewClarificationRequestDialog(JFrame parent, ProblemSet problems) {
		super(parent, Lang.getString("NewClarificationRequest"), true);
		this.actionSignal = new ActionEventGenerator();
		this.problems = problems;
		buildDialog();
		wire();
	}
	
	/** Initialization method that creates the dialog. */
	public void buildDialog() {
		Container contentPane = getContentPane();
		
		contentPane.setLayout(new GridBagLayout());
		GridBagConstraints selectionConstraints = new GridBagConstraints();
		GridBagConstraints textConstraints = new GridBagConstraints();
		GridBagConstraints buttonConstraints = new GridBagConstraints();
		
		selectionConstraints.gridx = 0;
		selectionConstraints.gridy = 0;
		selectionConstraints.weightx = 1;
		selectionConstraints.weighty = 0;
		selectionConstraints.fill = GridBagConstraints.BOTH;
		
		textConstraints.gridx = 0;
		textConstraints.gridy = 1;
		textConstraints.weightx = 1;
		textConstraints.weighty = 1;
		textConstraints.fill = GridBagConstraints.BOTH;
		
		buttonConstraints.gridx = 0;
		buttonConstraints.gridy = 2;
		buttonConstraints.weightx = 1;
		buttonConstraints.weighty = 0;
		buttonConstraints.fill = GridBagConstraints.BOTH;
		
		contentPane.add(getProblemSelection(), selectionConstraints);
		contentPane.add(getTextField(), textConstraints);
		contentPane.add(getButtons(), buttonConstraints);
		
		setSize(300,200);
	}
	
	
	/** Getter for the button panel containing the cancel and ok button.
	 * @return A JPanel containing the cancel and okay buttons.
	 */
	private JPanel getButtons() {
		JPanel buttons = new JPanel(new FlowLayout(FlowLayout.TRAILING));
		buttons.add(getCancelButton());
		buttons.add(getOKButton());
		return buttons;
	}

	/** Getter for the okay button.
	 * @return The okay button.
	 */
	private JButton getOKButton() {
		if(okButton == null) {
			okButton = new JButton(Lang.getString("OK"));
			okButton.setEnabled(false);
		}
		return okButton;
	}

	/**
	 * Getter for the cancel button.
	 * @return  The cancel button.
	 * @uml.property  name="cancelButton"
	 */
	private JButton getCancelButton() {
		if(cancelButton == null) {
			cancelButton = new JButton(Lang.getString("Cancel"));
		}
		return cancelButton;
	}

	/**
	 * Getter for the text field in which to write the request.
	 * @return  The JEditorPane text field.
	 * @uml.property  name="textField"
	 */
	private JEditorPane getTextField() {
		if(textField == null) {
			textField = new JEditorPane();
			textField.setContentType(Lang.getString("textplain"));
		}
		return textField;
	}

	/**
	 * Getter for the problem selection combo box.
	 * @return  Returns the problem selection box.
	 * @uml.property  name="problemSelection"
	 */
	private JComboBox getProblemSelection() {
		if(problemSelection == null) {
			ProblemWrapper[] entries = new ProblemWrapper[problems.size()+1];
			entries[0] = new ProblemWrapper(null);
			Iterator i = problems.iterator();
			int j = 1;
			while(i.hasNext()) {
				Problem p = (Problem) i.next();
				entries[j] = new ProblemWrapper(p);
				++j;
			}
			problemSelection = new JComboBox(entries);
		}
		return problemSelection;
	}

	/** Initialization method that registers EventListeners. */
	public void wire() {
		getTextField().getDocument().addDocumentListener(new DocumentListener() {

			public void changedUpdate(DocumentEvent arg0) {				
			}

			public void insertUpdate(DocumentEvent arg0) {
				getOKButton().setEnabled(true);
			}

			public void removeUpdate(DocumentEvent arg0) {
				getOKButton().setEnabled(arg0.getDocument().getLength() > 0);
			}
			
		});
		
		getCancelButton().addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				dispose();
			}
		});
		
		getOKButton().addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				dispose();
				actionSignal.emitActionPerformed(arg0);
			}
		});
	}

	/** Registers an ActionListener.	
	 * @param listener The listener to add.
	 */
	public void addActionListener(ActionListener listener) {
		actionSignal.registerListener(listener);
	}
	
	/** Get the currently selected problem. 
	 * @return Returns the problem that is currently selected, <code>null</code> if there is none, as this is intended to be a general question. 
	 */
	public Problem getSelectedProblem() {
		ProblemWrapper pw = (ProblemWrapper) getProblemSelection().getSelectedItem();
		return pw.getProblem();
	}
	
	/** Get the question in the text field. 
	 * @return Returns the text from the textField.
	 */
	public String getText() {
		return getTextField().getText();
	}
	
}
