/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.gui.judge;

import de.tudarmstadt.lang.Lang;
import de.tudarmstadt.yajs.gui.judge.events.RefreshScoreboardEvent;
import de.tudarmstadt.yajs.gui.judge.events.RefreshScoreboardEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.RefreshScoreboardEventListener;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.StringReader;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import org.jdom.Document;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.jdom.transform.XSLTransformException;
import org.jdom.transform.XSLTransformer;

/**
 * A panel showing the scoreboard.
 * @author  markus
 */
public class ScoreboardPanel extends JPanel {
	private RefreshScoreboardEventGenerator refreshSignal = new RefreshScoreboardEventGenerator();
	private JEditorPane viewer;
	private JButton refreshButton;
	private static final String errorPage = "<html><head></head><body><h1>Error transforming scoreboard</h1></body></html>"; //$NON-NLS-1$
	
	/** Default constructs the scoreboard panel.
	 *
	 */
	public ScoreboardPanel() {
		this.viewer = createEditor(); 
		this.refreshButton = createRefreshButton();
		buildPanel();
	}
	
	/** Build the panel.
	 * Sets up layout and adds components.
	 */
	private void buildPanel() {
		GridBagLayout layout = new GridBagLayout();
		setLayout(layout);
		
		GridBagConstraints c1 = new GridBagConstraints();
		c1.fill = GridBagConstraints.BOTH;
		c1.gridwidth = GridBagConstraints.REMAINDER;
		c1.weightx = 1;
		c1.weighty = 1;
		JScrollPane scrollpane = new JScrollPane(viewer);
		layout.setConstraints(scrollpane, c1);
		
		JPanel buttonbar = new JPanel(new FlowLayout(FlowLayout.TRAILING));
		buttonbar.add(refreshButton);
		c1.weighty = 0;
		layout.setConstraints(buttonbar, c1);
		
		add(scrollpane);
		add(buttonbar);
	}

	/** Create the refresh button.
	 * @return Returns the newly created refresh button.
	 */
	private JButton createRefreshButton() {
		JButton button = new JButton(Lang.getString("ScoreboardPanel.1")); //$NON-NLS-1$
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				refreshSignal.emitRefresh(new RefreshScoreboardEvent(this));
			}
		});
		return button;
	}

	/** Create the JEditorPane (HTML view).
	 * @return Returns the newly created editor.
	 */
	private JEditorPane createEditor() {
		JEditorPane editor = new JEditorPane();
		editor.setEditable(false);
		editor.setContentType("text/html"); //$NON-NLS-1$
		editor.setPreferredSize(new Dimension(300,200));
		return editor;
	}

	/** Set the scoreboard.
	 * Expects the implementation-dependent XML document, which is 
	 * transformed into a suitable (X)HTML presentation.
	 * @param text The scoreboard.
	 */
	public void setText(String text) {
		final String doc = transformToHTML(text);
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				viewer.setDocument(viewer.getEditorKit().createDefaultDocument());
				viewer.setText(doc);
			}
		});
	}
	
	/** Transforms the incoming XML document to HTML.
	 * @param text
	 * @return
	 */
	private String transformToHTML(String text) {
		try {
			StringReader input = new StringReader(text);
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(input);
			XSLTransformer transformer = new XSLTransformer("scoreboard.xsl"); //$NON-NLS-1$
			Document htmldoc = transformer.transform(doc);
			Format htmlformat = Format.getCompactFormat();
			htmlformat.setExpandEmptyElements(true);
			return new XMLOutputter(htmlformat).outputString(htmldoc.getRootElement());
		} catch (XSLTransformException e) {
			// e.printStackTrace(); 
		} catch (JDOMException e) {
			// e.printStackTrace();
		} catch (IOException e) {
			// e.printStackTrace();
		}
		return errorPage;
	}

	/** Add a RefreshScoreboardEventListener.
	 * @param listener The listener.
	 */
	public void addRefreshScoreboardListener(RefreshScoreboardEventListener listener) {
		refreshSignal.registerListener(listener);
	}
}
