/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.gui.judge;

import de.tudarmstadt.events.EventGenerator;
import de.tudarmstadt.lang.Lang;
import de.tudarmstadt.yajs.gui.FilteredMapItemSelection;
import de.tudarmstadt.yajs.gui.MyTableItemLayout;
import de.tudarmstadt.yajs.gui.MyTableModel;
import de.tudarmstadt.yajs.gui.Predicate;
import de.tudarmstadt.yajs.gui.ResultPrinter;
import de.tudarmstadt.yajs.gui.judge.events.RefreshSubmissionsEvent;
import de.tudarmstadt.yajs.gui.judge.events.RefreshSubmissionsEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.RefreshSubmissionsEventListener;
import de.tudarmstadt.yajs.gui.judge.events.SubmissionSelectedEvent;
import de.tudarmstadt.yajs.gui.judge.events.SubmissionSelectedEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.SubmissionSelectedEventListener;
import de.tudarmstadt.yajs.lib.JudgeResult;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;

/**
 * A panel showing submissions in a table. A filter can be applied by specifying a Predicate.
 * @author  markus
 */
public class SubmissionsPanel extends JPanel {

	private JudgeUIModel model;
	private JTable judgementstable;
	private EventGenerator submissionselectedsignal; 
	private EventGenerator refreshsubmissionssignal;
	
	/** Constructor.
	 * @param model The judge ui model.
	 * @param filter The filter predicate.
	 */
	public SubmissionsPanel(JudgeUIModel model, Predicate filter) {
		super();
		this.model = model;
		this.submissionselectedsignal = new SubmissionSelectedEventGenerator();
		this.refreshsubmissionssignal = new RefreshSubmissionsEventGenerator();
		buildPanel(filter);
	}

	/** Initialization method that creates the panel. 
	 * @param filter The filter to use.
	 */
	private void buildPanel(Predicate filter)
	{
		/* Initialize layout (grid bag layout) */
		
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		setLayout(gbl);
		
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		
		gbc.weightx = gbc.weighty = 1.0;
		
		/* initialize the result table */

		MyTableItemLayout judgementlayout = new MyTableItemLayout() {

			public int getColumnCount() {
				return 7;
			}

			public String getColumnName(int col) {
				switch(col)
				{
					case 0: return "#"; //$NON-NLS-1$
					case 1: return Lang.getString("SubmissionsPanel.1"); //$NON-NLS-1$
					case 2: return Lang.getString("SubmissionsPanel.2"); //$NON-NLS-1$
					case 3: return Lang.getString("SubmissionsPanel.3"); //$NON-NLS-1$
					case 4: return Lang.getString("SubmissionsPanel.4"); //$NON-NLS-1$
					case 5: return Lang.getString("SubmissionsPanel.5"); //$NON-NLS-1$
					case 6: return Lang.getString("SubmissionsPanel.6"); //$NON-NLS-1$
				}
				return ""; //$NON-NLS-1$
			}

			public Object getValue(Object item, int col) {
				JudgeResult r = (JudgeResult) item;
				switch(col)
				{
					case 0: return new Long(r.getIndex());
					case 1: return r.getContestant();
					case 2: return r.getProblem().getID() + " - " + r.getProblem().getName(); //$NON-NLS-1$
					case 3: return r.getLanguage().getName();
					case 4: return new Long(r.getTime());
					case 5: return ResultPrinter.getVerdict(r);
					case 6: 
						if(r.getLock().equals("")) return r.getJudge(); //$NON-NLS-1$
						else return Lang.getString("SubmissionsPanel.10"); //$NON-NLS-1$
					default: return Lang.getString("SubmissionsPanel.11"); //$NON-NLS-1$
				}
			}
		};

		judgementstable = new JTable( new MyTableModel(getModel().getJudgements(), judgementlayout, new FilteredMapItemSelection(getModel().getJudgements(), filter)) );
		judgementstable.addMouseListener(new MouseAdapter() {
			/* (non-Javadoc)
			 * @see java.awt.event.MouseAdapter#mouseClicked(java.awt.event.MouseEvent)
			 */
			public void mouseClicked(MouseEvent arg0) {
				super.mouseClicked(arg0);
				if(arg0.getClickCount() > 1) mouseDoubleClicked(arg0);
			}
			
			public void mouseDoubleClicked(MouseEvent arg0)
			{
				submissionSelected();
			}
		});
		judgementstable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		JScrollPane runsscrollpane = new JScrollPane(judgementstable);
		
		runsscrollpane.setBorder(BorderFactory.createTitledBorder(Lang.getString("SubmissionsPanel.12"))); //$NON-NLS-1$
		
		gbl.setConstraints(runsscrollpane, gbc);
		
		add(runsscrollpane);
		
		/* Add buttons */
		
		gbc.weightx = gbc.weighty = 0.0;
		
		JPanel buttons = new JPanel();
		buttons.setLayout(new FlowLayout(FlowLayout.TRAILING));
		JButton refreshbutton = new JButton(Lang.getString("SubmissionsPanel.13")); //$NON-NLS-1$
		buttons.add(refreshbutton);
		refreshbutton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				refreshsubmissionssignal.emitEvent( new RefreshSubmissionsEvent(SubmissionsPanel.this));
			}
		});
		gbl.setConstraints(buttons, gbc);
		add(buttons);
	}
	
	
	/**
	 * Retrieve the underlying judge ui model.
	 * @return  Returns the JudgeUIModel.
	 * @uml.property  name="model"
	 */
	private JudgeUIModel getModel() {
		return model;
	}
	
	/** Callback when a submission was selected. */
	private void submissionSelected()
	{
		MyTableModel tm = (MyTableModel) getJudgementsTable().getModel(); 
		if(getJudgementsTable().getSelectedRow() == -1) return;
		JudgeResult result = (JudgeResult) tm.getItem(getJudgementsTable().getSelectedRow()); 
		long nr = result.getIndex();
		submissionselectedsignal.emitEvent(new SubmissionSelectedEvent(this, nr));
	}


	/** Registers a RefreshSubmissionsEventListener that will be called back whenever the user wants to refresh the list of submissions.
	 * @param listener The listener to register.
	 */
	public void addRefreshSubmissionsListener(RefreshSubmissionsEventListener listener) {
		refreshsubmissionssignal.registerListener(listener);
	}


	/** Registers a SubmissionSelectedEventListener that will be called back whenever the user has selected a submission in order to judge it.
	 * @param listener
	 */
	public void addSubmissionSelectedEventListener(SubmissionSelectedEventListener listener) {
		submissionselectedsignal.registerListener(listener);
	}
	
	/** Getter for the table containing the list of submissions along with judgements.
	 * @return Returns the judgementstable.
	 */
	public JTable getJudgementsTable() {
		return judgementstable;
	}
}
