/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.gui.judge;

import de.tudarmstadt.lang.Lang;
import de.tudarmstadt.yajs.gui.Predicate;
import de.tudarmstadt.yajs.gui.ResultPrinter;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEvent;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEventListener;
import de.tudarmstadt.yajs.gui.judge.events.ClarificationSelectedEvent;
import de.tudarmstadt.yajs.gui.judge.events.ClarificationSelectedEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.ClarificationSelectedEventListener;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEvent;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEventListener;
import de.tudarmstadt.yajs.gui.judge.events.RefreshScoreboardEvent;
import de.tudarmstadt.yajs.gui.judge.events.RefreshScoreboardEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.RefreshScoreboardEventListener;
import de.tudarmstadt.yajs.gui.judge.events.RefreshServerStatusEventListener;
import de.tudarmstadt.yajs.gui.judge.events.RefreshSubmissionsEventListener;
import de.tudarmstadt.yajs.gui.judge.events.SubmissionSelectedEvent;
import de.tudarmstadt.yajs.gui.judge.events.SubmissionSelectedEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.SubmissionSelectedEventListener;
import de.tudarmstadt.yajs.gui.judge.events.TogglePauseEventListener;
import de.tudarmstadt.yajs.lib.ContestStatus;
import de.tudarmstadt.yajs.lib.JudgeResult;
import de.tudarmstadt.yajs.lib.Status;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;

/**
 * The user interface for the judge client (i.e. <em>sage</em>).
 * @author  markus
 */
public class YAJSJudgeUI extends JFrame {
	private SubmissionsPanel submissionPanel; 
	private SubmissionsPanel historyPanel;
	private ClarificationsPanel clarificationsPanel;
	private XMLDisplayPanel scoreboardPanel;
	private MiscellaneousPanel miscPanel;
	
	protected SubmissionSelectedEventGenerator submissionSelectedSignal = new SubmissionSelectedEventGenerator();
	protected ClarificationSelectedEventGenerator clarificationSelectedSignal = new ClarificationSelectedEventGenerator();
	protected AskClarificationEventGenerator askClarificationSignal = new AskClarificationEventGenerator();
	protected RefreshClarificationsEventGenerator refreshClarificationsSignal = new RefreshClarificationsEventGenerator();
	protected RefreshScoreboardEventGenerator refreshScoreboardSignal = new RefreshScoreboardEventGenerator();
	private JudgeUIModel model;
	
	/** Constructorr
	 * @param model The judgeui model.
	 */
	public YAJSJudgeUI(JudgeUIModel model) {
		
		super();
		
		this.model = model;
		
		submissionPanel = new SubmissionsPanel( model, new Predicate(){
			public boolean contains(Object element) {
				JudgeResult r = (JudgeResult) element;
				return r.needsJudgement();
			}
		});

		historyPanel = new SubmissionsPanel( model, Predicate.alwaysTrue );
		
		SubmissionSelectedEventListener submissionSelectedListener = new SubmissionSelectedEventListener() {
			public void handleSubmission(SubmissionSelectedEvent sse) {
				submissionSelectedSignal.emitEvent(sse);
			}
		};

		submissionPanel.addSubmissionSelectedEventListener(submissionSelectedListener);
		historyPanel.addSubmissionSelectedEventListener(submissionSelectedListener);
		
		clarificationsPanel = new ClarificationsPanel(model, Predicate.alwaysTrue);
		clarificationsPanel.addClarificationSelectedEventListener(new ClarificationSelectedEventListener() {
			public void clarificationSelected(ClarificationSelectedEvent e) {
				clarificationSelectedSignal.emitClarificationSelected(e);
			}
		});
		clarificationsPanel.addAskClarificationEventListener(new AskClarificationEventListener() {
			public void askClarification(AskClarificationEvent e) {
				askClarificationSignal.emitAskClarification(e);
			}
		});
		clarificationsPanel.addRefreshClarificationsEventListener(new RefreshClarificationsEventListener() {
			public void refreshClarifications(RefreshClarificationsEvent e) {
				refreshClarificationsSignal.emitRefreshClarifications(e);
			}
		});
		
		scoreboardPanel = new XMLDisplayPanel("scoreboard.xsl"); //$NON-NLS-1$
		scoreboardPanel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				refreshScoreboardSignal.emitRefresh(new RefreshScoreboardEvent(arg0.getSource()));
			} 
		});
		
		miscPanel = new MiscellaneousPanel();
		
		JTabbedPane tp = new JTabbedPane();
		
		tp.addTab(Lang.getString("YAJSJudgeUI.1"), null, submissionPanel, Lang.getString("YAJSJudgeUI.2")); //$NON-NLS-1$ //$NON-NLS-2$
		tp.addTab(Lang.getString("YAJSJudgeUI.3"), null, historyPanel, Lang.getString("YAJSJudgeUI.4")); //$NON-NLS-1$ //$NON-NLS-2$
		tp.addTab(Lang.getString("YAJSJudgeUI.5"), null, clarificationsPanel, Lang.getString("YAJSJudgeUI.6")); //$NON-NLS-1$ //$NON-NLS-2$
		tp.addTab(Lang.getString("YAJSJudgeUI.7"), null, scoreboardPanel, Lang.getString("YAJSJudgeUI.8")); //$NON-NLS-1$ //$NON-NLS-2$
		tp.addTab(Lang.getString("YAJSJudgeUI.9"), null, miscPanel, Lang.getString("YAJSJudgeUI.10")); //$NON-NLS-1$ //$NON-NLS-2$
		
		// TODO only insert pause button if pause supported.
		getContentPane().add(tp);
		pack();
	}

	/** Show an updated judgement. This will pop up a message box.
	 * @param judgement The judgement to show.
	 */
	public void showUpdatedJudgement(JudgeResult judgement) {
		String message = Lang.getString("YAJSJudgeUI.11")  //$NON-NLS-1$
		+ judgement.getProblem().getID() 
		+ " - '" + judgement.getProblem().getName() //$NON-NLS-1$
		+ Lang.getString("YAJSJudgeUI.13") //$NON-NLS-1$
		+ judgement.getContestant()
		+ "':\n\n"  //$NON-NLS-1$
		+ ResultPrinter.getVerdict(judgement);
		String title = Lang.getString("YAJSJudgeUI.15") + judgement.getTime() + Lang.getString("YAJSJudgeUI.16"); //$NON-NLS-1$ //$NON-NLS-2$
		JOptionPane.showMessageDialog(this, message, title, JOptionPane.INFORMATION_MESSAGE);
	}

	/** Registers a RefreshSubmissionsListener that will be called back whenever the user wants to refresh the list of submissions.
	 * @param listener The listener to register.
	 */
	public void addRefreshSubmissionsListener(RefreshSubmissionsEventListener listener) {
		submissionPanel.addRefreshSubmissionsListener(listener);
	}
	
	/** Registers a RefreshScoreboardListener that will be called back whenever the user wants to refresh the scoreboard.
	 * @param listener The listener to register.
	 */
	public void addRefreshScoreboardListener(RefreshScoreboardEventListener listener) {
		refreshScoreboardSignal.registerListener(listener);
	}

	/** Registers a RefreshSubmissionsEventListener that will be called back whenever the user wants to refresh the list of submissions.
	 * @param listener The listener to register.
	 */
	public void addSubmissionSelectedEventListener(SubmissionSelectedEventListener listener) {
		submissionSelectedSignal.registerListener(listener);
	}
	
	/** Registers a ClarificationSelectedEventListener that will be called back whenever the user has selected a clarification request in order to answer it.
	 * @param listener The listener to register.
	 */
	public void addClarificationSelectedEventListener(ClarificationSelectedEventListener listener) {
		clarificationSelectedSignal.registerListener(listener);
	}

	/** Registers a AskClarificationEventListener that will be called back whenever the user wants to ask for clarification.
	 * @param listener The listener to register.
	 */
	public void addAskClarificationEventListener(AskClarificationEventListener listener) {
		askClarificationSignal.registerListener(listener);
	}
	
	/** Registers a RefreshClarificationsEventListener that will be called back whenever the user wants to refresh the list of clarifications.
	 * @param listener The listener to register.
	 */
	public void addRefreshClarificationsEventListener(RefreshClarificationsEventListener listener) {
		refreshClarificationsSignal.registerListener(listener);
	}
	
	/** Registers a RefreshServerStatusListener that will be called back whenever the user wants to refresh the server status.
	 * @param listener The listener to register.
	 */
	public void addRefreshServerStatusEventListener(RefreshServerStatusEventListener listener) {
		miscPanel.addRefreshServerStatusEventListener(listener);
	}
	
	
	/** Shows a judgement dialog where a judge can test a submission and select a judgement.
	 * <b>REFACTOR</b> should only need id.
	 * @param id The id of the submission to judge.
	 * @param source The source code in the submission.
	 */
	public void openJudgementDialog(long id, String source) {
		JDialog dlg = new JudgementDialog(getModel().getOwnName(), getModel().getJudgement(id), source);
		dlg.show();
	}

	/**
	 * Getter for the judge ui model.
	 * @return  Returns the JudgeUIModel.
	 * @uml.property  name="model"
	 */
	private JudgeUIModel getModel() {
		return model;
	}

	/** Shows an error message.
	 * @param string The error message to display.
	 */
	public void showErrorMessage(final String string) {
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				public void run() {
					JOptionPane.showMessageDialog(YAJSJudgeUI.this, string, Lang.getString("YAJSJudgeUI.17"), JOptionPane.ERROR_MESSAGE);		 //$NON-NLS-1$
				}	
			});
		} catch (Exception e) {
			e.printStackTrace();
			System.err.println(Lang.getString("YAJSJudgeUI.18") + string); //$NON-NLS-1$
		}
	}

	/** Registers a TogglePauseEventListener that will be called back whenever the user wants switch between contest states.
	 * <b>REFACTOR</b> Change name, name is "toggle" only for historical reasons.
	 * @param listener The listener to register.
	 */
	public void addTogglePauseEventListener(TogglePauseEventListener listener) {
		miscPanel.addTogglePauseEventListener(listener);
	}

	/** Update the contest status that is displayed.
	 * @param contestStatus The current contest status.
	 */
	public void updateContestStatus(ContestStatus contestStatus) {
		miscPanel.updateContestStatus(contestStatus);
	}

	/** Update the scoreboard. The scoreboard is given as an XML document.
	 * @param scoreboard The scoreboard to display.
	 */
	public void updateScoreboard(String scoreboard) {
		scoreboardPanel.setText(scoreboard);
	}

	/** Update the server status.
	 * @param status The server status.
	 */
	public void updateServerStatus(String status) {
		miscPanel.updateServerStatus(status);
	}

}
