/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on Dec 5, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package de.tudarmstadt.yajs.lib;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.File;
import java.net.InetSocketAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Map;

/**
 * The contest information. Contains the set of problems, the set of languages and information from the configuration.
 * @author  markus
 */
public class ContestInfo {

	private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
	
	private InetSocketAddress server;
	
	private LanguageSet languages;
	private ProblemSet problems;
	private String guiLanguage;
	private URI testBase;
	private Map languageAliases;
	private String mode;
	

	
	/** Constructor. Initializes fields from given values.
	 * @param server The server's address.
	 * @param languages The set of languages.
	 * @param problems The set of problems.
	 * @param testBase The URI under which test data can be found.
	 * @param languageAliases The map from language aliases to languages.
	 * @param language True if the system user-name should be used as login name and password.
	 */
	public ContestInfo(InetSocketAddress server, LanguageSet languages, ProblemSet problems) {
		this.server = server;
		this.languages = languages;
		this.problems = problems;
	}
	public ContestInfo(InetSocketAddress server, LanguageSet languages, ProblemSet problems, URI testBase, String guiLanguage, String mode) {
		this(server, languages, problems);
		this.testBase = testBase;
		this.guiLanguage=guiLanguage;
		this.mode=mode;
		
	}
	
	/**
	 * Getter for server.
	 * @return  Returns the InetSocketAddress of the server.
	 * @uml.property  name="server"
	 */
	public InetSocketAddress getServer() {
		return server;
	}

	/**
	 * Getter for problems.
	 * @return  Returns the problems.
	 * @uml.property  name="problems"
	 */
	public ProblemSet getProblems() {
		return problems;
	}
	
	/**
	 * Getter for languages.
	 * @return  Returns the languages.
	 * @uml.property  name="languages"
	 */
	public LanguageSet getLanguages() {
		return languages;
	}


	/**
	 * Getter for testBase.
	 * @return  Returns the testBase.
	 * @uml.property  name="testBase"
	 */
	public URI getTestBase() {
		return testBase;
	}
	
	/**
	 * Setter for server.
	 * @param server  The server to set.
	 * @uml.property  name="server"
	 */
	public void setServer(InetSocketAddress server) {
		InetSocketAddress old = this.server;
		this.server = server;
		if(server.equals(this.server)) return;
		pcs.firePropertyChange("server", old, server);
	}
	
	/**
	 * Setter for languageAliases.
	 * @param languageAliases  The languageAliases to set.
	 * @uml.property  name="languageAliases"
	 */
	public void setLanguageAliases(Map languageAliases) {
		Map old = this.languageAliases;
		this.languageAliases = languageAliases;
		if(old.equals(languageAliases)) return;
		pcs.firePropertyChange("languageAliases", old, languageAliases);
	}
	
	/**
	 * Setter for languages.
	 * @param languages  The language set.
	 * @uml.property  name="languages"
	 */
	public void setLanguages(LanguageSet languages) {
		if(this.languages.equals(languages)) return;
		LanguageSet old = this.languages;
		this.languages = languages;
		pcs.firePropertyChange("languages", old, languages);
	}

	/**
	 * Setter for problems. 
	 * @param problems  The problem set.
	 * @uml.property  name="problems"
	 */
	public void setProblems(ProblemSet problems) {
		if(this.problems.equals(problems)) return;
		ProblemSet old = this.problems;
		this.problems = problems;
		pcs.firePropertyChange("problems", old, problems);
	}
	
	/** Add a PropertyChangeListener.
	 * @param arg0 The listener.
	 */
	public void addPropertyChangeListener(PropertyChangeListener arg0) {
		pcs.addPropertyChangeListener(arg0);
	}
	/** Add a PropertyChangeListener.
	 * @param arg0 The property to associate the listener with.
	 * @param arg1 The listener.
	 */
	public void addPropertyChangeListener(String arg0,
			PropertyChangeListener arg1) {
		pcs.addPropertyChangeListener(arg0, arg1);
	}
	/** Remove a PropertyChangeListener.
	 * @param arg0 The listener to remove.
	 */
	public void removePropertyChangeListener(PropertyChangeListener arg0) {
		pcs.removePropertyChangeListener(arg0);
	}
	/** Remove a PropertyChangeListener.
	 * @param arg0 The property the listener is associated with.
	 * @param arg1 The listener to remove.
	 */
	public void removePropertyChangeListener(String arg0,
			PropertyChangeListener arg1) {
		pcs.removePropertyChangeListener(arg0, arg1);
	}
	
	/** Get the language an alias denotes.
	 * @param aliasName an alias.
	 * @return Returns the language corresponding to aliasName. 
	 */
	public Language getLanguageForAlias(String aliasName) {
		String realName = (String) languageAliases.get(aliasName);
		if(realName == null) return null;
		return getLanguages().getLanguage(realName);
	}
	
	/** Get the File denoting the local location (directory) of the test files for problem <tt>problem</tt>
	 * @param problem A problem.
	 * @return Returns the File denoting the local test location for the given problem.
	 */
	public File getLocalTestLocation(Problem problem) {
		return new File(problem.getID());
	}
	
	/** Get the local test input data for a problem.
	 * @param problem The problem to get the input for.
	 * @return Returns a File describing the input.
	 */
	public File getLocalTestInputLocation(Problem problem) {
		return new File(getLocalTestLocation(problem), "sin");
	}
	public File getLocalTestZipLocation(Problem problem){
		return new File(problem.getID()+".zip");
	}
	public File getLocalTestTemplateLocation(Problem problem, Language language){
		return new File(problem.getID(),problem.getID()+"."+language.getID());
	}
	/** Get the local test output data for a problem.
	 * @param problem The problem to get the output for.
	 * @return Returns a File describing the output.
	 */
	public File getLocalTestOutputLocation(Problem problem) {
		return new File(getLocalTestLocation(problem), "sout");
	}
	
	/** Get the URI denoting the remote location of the test files for problem <tt>problem</tt>
	 * @param problem A problem.
	 * @return Returns the URI denoting the remote test location for the given problem.
	 * @throws URISyntaxException
	 */
	public URI getRemoteTestLocation(Problem problem) throws URISyntaxException {
		return testBase.resolve(new URI(problem.getID() + "/"));
	}
	/** Get the URI denoting the remote location of the test files for problem <tt>problem</tt>
	 * @param problem A problem.
	 * @return Returns the URI denoting the remote test location for the given problem.
	 * @throws URISyntaxException
	 */
	public URI getRemoteTestZipLocation(Problem problem) throws URISyntaxException {
		return testBase.resolve(new URI(problem.getID() + ".zip"));
	}
	
	/**
	 *  Get the remote test input for a problem. 
	 * @param problem The problem to get the input for.
	 * @return Returns a URI describing the input.
	 */
	public URI getRemoteTestInputLocation(Problem problem) {
		try {
			return getRemoteTestLocation(problem).resolve("sin");
		} catch (URISyntaxException e) {
			System.err.println("Fatal _BUG_ while building remote input location!");
			System.exit(1);
			return null;
		}
	}
	
	/** Get the remote test output for a problem. 
	 * @param problem The problem to get the output for.
	 * @return Returns a URI describing the output.
	 */
	public URI getRemoteTestOutputLocation(Problem problem) {
		try {
			return getRemoteTestLocation(problem).resolve("sout");
		} catch (URISyntaxException e) {
			System.err.println("Fatal _BUG_ while building remote output location!");
			System.exit(1);
			return null;
		}
	}

	public String getGuiLanguage() {
		// TODO Auto-generated method stub
		return guiLanguage;
	}
	public String getMode(){
		return mode;
		
	}

	
	
}
