/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.lib;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.*;
import java.nio.channels.*;

/**
 * The thread receiving messages from a socket and pushing them in a queue to be read from.
 */
class MessageReceiver extends java.lang.Thread
{
	private MessageQueue q;
	private SocketChannel channel;
	private boolean term;
	private boolean died;
	
	/** Constructor. Creates a MessageReceiver writing messages to <code>queue</code> when they have been read from <code>c</code>.
	 * @param queue The MessageQueue to push incoming messages to.
	 * @param c The SocketChannel to read from.
	 */
	public MessageReceiver(MessageQueue queue, SocketChannel c) throws IOException
	{
		super("MessageReceiver");
		q = queue;
		channel = c;
		channel.configureBlocking(true);
		term = false;
		died = false;
	}
	
	/** Find out whether the thread is meant to terminate. 
	 * @return Returns true if the thread should terminate.
	 */
	public synchronized boolean isTerminated()
	{
		return term;
	}
	
	/** Set the termination flag. This will effectively stop the thread. */
	public synchronized void terminate()
	{
		term = true;
	}
	
	/** Find out whether the thread has died, either because of an exception or because it was terminated.
	 * @return Returns true if the thread has died.
	 */
	public synchronized boolean hasDied()
	{
		return died;
	}
	
	/** The run method. Reads from the socket and pushes completed messages to the queue.
	 */
	public void run()
	{
		try
		{
			while(! isTerminated())
			{
				ByteBuffer szbuf = ByteBuffer.allocate(10);
				long bytesread = 0;	
				while(bytesread < 10)
				{
					if(isTerminated()) return;
					int err = channel.read(szbuf);
					if(err == -1) throw new IOException("Connection closed by foreign host."); 
					bytesread += err;
				}
				szbuf.rewind();
				String a = new String(szbuf.array(), "ASCII");
				String b = a.trim();
				long size = Long.parseLong(b);
				
				bytesread = 0;
				ByteBuffer[] message = new ByteBuffer[] { ByteBuffer.allocate((int)(size/2)), ByteBuffer.allocate((int)(size - size/2)) };
				while(bytesread < size)
				{
					if(isTerminated()) return;
					long err = channel.read(message);
					if(err == -1) throw new IOException("Connection closed by foreign host.");
					bytesread += err;
				}
				String msg1 = new String(message[0].array(), "UTF-8");
				String msg2 = new String(message[1].array(), "UTF-8");
				String msg = msg1 + msg2;
			//	System.err.println("[" + msg + "]");
				q.push(msg.toString());
			}
		}
		catch(AsynchronousCloseException e)
		{
			if(!isTerminated()) System.err.println("Socket closed but Receiver not terminated? Oops!");
		}
		catch(IOException e)
		{
			System.err.println("IO Exceeption in MessageReceiver:");
			System.err.println(e);
		}
		finally
		{
			if(!isTerminated())
			{
				try {
					// TODO handle case where no "error" message was received...
					synchronized(q)
					{
						while(q.size() > 0)	q.wait();
					}
				} catch (InterruptedException e1) {
					e1.printStackTrace();
				}
			}
			synchronized(this) { died = true; }
		}
	}
	
	/** Only used for testing. */
	public static void main(String[] args) throws Exception
	{
		SocketChannel sc = SocketChannel.open();
		sc.connect(new InetSocketAddress("localhost", 27251));
		MessageQueue q = new MessageQueue();
		MessageReceiver mp = new MessageReceiver(q, sc);
		try
		{
			System.err.println("MessageProducer Tester");
			mp.start();
			while(true)
			{
				if(q.size() > 0)
				{
					String m = q.pop();
					System.err.println("Message extracted:\n" + m);
				}
				yield();
			}
		}
		finally
		{
			mp.terminate();
			mp.join();
			sc.close();
		}
	}
}
