/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.lib;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.AsynchronousCloseException;
import java.nio.channels.SocketChannel;

/**
 * The thread that sends messages waiting in a queue.
 */
class MessageSender extends java.lang.Thread
{
	private MessageQueue q;
	private SocketChannel channel;
	private boolean term;
	private boolean died;
	
	/** Constructor. Create a sender thread reading from <code>queue</code> and writing to the socket <code>c</code>.
	 * @param queue The queue where messages to be sent are stored.
	 * @param c The SocketChannel to write to. 
	 */
	public MessageSender(MessageQueue queue, SocketChannel c) throws IOException
	{
		super("MessageSender");
		q = queue;
		channel = c;
		channel.configureBlocking(true);
		term = false;
		died = false;
	}
	
	/** Test whether the thread is meant to be terminated. 
	 * @return Returns true if the thread should terminate.
	 */
	public synchronized boolean isTerminated()
	{
		return term;
	}
	
	/** Set the termination flag. 
	 * This will effectively stop the thread.
	 */
	public synchronized void terminate()
	{
		term = true;
	}
	
	/** Find out whether the thread has died, either because of an exception or because it was terminated.
	 * @return Returns true if the thread isn't running any more.
	 */
	public synchronized boolean hasDied()
	{
		return died;
	}
	
	/** The run method of the thread.
	 * Waits for messages to arrive in the queue. Sends any messages arriving.
	 */
	public void run()
	{
		try
		{
			while(! isTerminated())
			{
				String msg;
				synchronized(q)
				{
					while(q.size() == 0) { if(isTerminated()) return; q.wait(250); }
					msg = q.pop();
				}
				
				byte[] msg_array = msg.getBytes("UTF-8");
				StringBuffer len = new StringBuffer();
				len.append(msg_array.length);
				len.append("          ");
				len.setLength(10);
				byte[] len_array = len.toString().getBytes("ASCII");
				
				ByteBuffer outbuf = ByteBuffer.allocate(len_array.length + msg_array.length);
				outbuf.put(len_array);
				outbuf.put(msg_array);
				outbuf.rewind();
				channel.write(outbuf);
			}
		}
		catch(AsynchronousCloseException e)
		{
			System.err.println("MessageSender: Socket closed");
		}
		catch(Exception e)
		{
			System.err.println("Exception in MessageSender:");
			e.printStackTrace();
			System.err.println(e);
		}
		finally
		{
			synchronized(this) { died = true; }
		}
	}	
}
