/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.lib;
import java.io.IOException;
import java.net.*;
import java.nio.channels.*;

/**
 * A message socket. This socket wraps all data into messages as defined by the protocol.
 */
class MessageSocket
{
	private SocketChannel sc;
	private MessageReceiver mr;
	private MessageSender ms;
	private MessageQueue q_recv;
	private MessageQueue q_send;
	
	/** Constructor. Create a message socket and connect to <code>addr</code>.
	 * It also starts two threads, one that reads messages from the socket, and one that writes methods to it.
	 */
	public MessageSocket(SocketAddress addr) throws IOException
	{
		sc = SocketChannel.open(addr);
		q_recv = new MessageQueue();
		q_send = new MessageQueue();
		mr = new MessageReceiver(q_recv,sc);
		ms = new MessageSender(q_send,sc);
		mr.start();
		ms.start();
	}

	/**
	 * When we close the connection, both sender and receiver are killed.
	 * When the connection is closed, the receiver might wait.
	 * So we are alive until the receiver has died.
	 * THIS IS VITAL!
	 * @return Returns <code>true</code> if the connection is still open.
	 */
	public boolean isAlive()
	{
		boolean alive = true;
		synchronized(mr) { alive &= !mr.hasDied(); }
		//synchronized(ms) { alive &= !ms.hasDied(); }
		return alive;
	}
	
	/** Send a message.
	 * Messages will be queued to be read by the sender thread.
	 */
	public void send(String msg)
	{
		q_send.push(msg);
	}

	/** Query the number of available messages, i.e. messages waiting to be read.
	 * @return Returns the number of messages that can be read.
	 */
	public int available() 
	{
		return q_recv.size();
	}
	
	/** Read a message. <b>Important:</b> Messages can only be read when <code>available()>0</code>.
	*/
	public String receive()
	{
		return q_recv.pop();
	}
	
	/** Close the connection.
	 * This closes the socket and waits for both threads to terminate.
	 */
	public void close() throws IOException
	{
		try
		{
			// TODO I think a ThreadGroup would be more appropriate here...
			boolean senderTerminated = false;
			synchronized(ms) { senderTerminated = ms.hasDied(); }
			if(!senderTerminated)
			{
				ms.terminate();
				ms.join();
				//System.err.println("MS dead");
			}
			boolean receiverTerminated = false;
			synchronized(mr) { receiverTerminated = mr.hasDied(); }
			if(!receiverTerminated)
			{
				mr.terminate();
				sc.socket().close();
				mr.join();
				//System.err.println("MR dead");
			}
		}
		catch(InterruptedException e)
		{
			System.err.println("Interrupted? Eh?\n" + e);
		}
	}
	
	/** This method was only used for testing. */
	public static void main(String[] args)
	{
		MessageSocket s = null;
		try
		{
			s = new MessageSocket(new InetSocketAddress("localhost", 27251));
			outer: while(true)
			{
				Thread.yield();
				while(s.available() > 0) 
				{
					String msg = s.receive();
					System.out.println("Got Message: " + msg);
					s.send("Why!\nThank you very much!");
					if(msg.equals("quit")) break outer;
				}
			}
		}
		catch(IOException e)
		{
			System.err.println("Ooops... IOException in MessageSocket\n" + e);
		}
		finally
		{
			try
			{
				s.close();
			}
			catch(IOException e)
			{
				System.err.println(e);
			}
		}
		System.err.println("Bye bye");
	}
}
