/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 30.03.2005
 *
 */
package de.tudarmstadt.yajs.tester;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JOptionPane;

import de.tudarmstadt.lang.Lang;
import de.tudarmstadt.yajs.Dcc32Utility;
import de.tudarmstadt.yajs.lib.CCompiler;
import de.tudarmstadt.yajs.lib.ContestInfo;
import de.tudarmstadt.yajs.lib.Language;
import de.tudarmstadt.yajs.lib.Problem;

/**
 * Testing facilities. This class offers separation of a thread that runs and 
 * tests a given program. As checking for time limits and compiling is highly
 *  platform dependent, it calls another  program which is named <code>native</code>. 
 *  It compiles and runs the program to test and writes its output to files specified 
 *  on the command line. This program is invoked using the following list of parameters:
 *   <ol> <li>The id of the programming language the given file is written in</li> 
 *   <li>The timeLimit as a floating point value (in seconds)</li> <li>The file to test</li> 
 *   <li>The path to the file which contains the standard input for the program</li> 
 *   <li>The path to the file to which the program's output will be written</li>  
 *   <li>The path to the file to which the program's error output will be written</li> </ol>
 * @author  markus
 */
public class Tester extends Thread {
		
	private static String TESTCOMMAND ;
	private Process testprocess;
	private TestCompletedEventGenerator testCompletedSignal;
	private String[] command;

	private File fileToTest;
	private File testIn;
	private File testOut;
	private File testErr;
	
	private boolean term;
	private boolean stdoutOpen;
	private boolean stderrOpen;

	/** Constructor. 
	 * Builds a tester for the submission <code>code</code>, the given <code>testInput</code>, the given <code>timeLimit</code> and the specified language and problem.
	 * Also creates temporary files for <code>stderr</code> and <code>stdout</code> output.
	 */
	public Tester(Problem problem, Language language, double timeLimit, String code, File testInput, ContestInfo contestinfo) throws IOException
	{
		this.testCompletedSignal = new TestCompletedEventGenerator();
		this.testIn = testInput;
		//add a debugging info such compiler type with compiler options.
		CCompiler compiler=language.getCompiler();
		//System.err.println("Compiling with "+contestinfo.getLanguages().getLanguage(language.getID()).getCompiler().toString()); //$NON-NLS-1$		
		System.err.println("Compiling with "+compiler.toString()); //$NON-NLS-1$		
		
		System.err.println(Lang.getString("Tester.2") + testIn); //$NON-NLS-1$
		this.fileToTest = createTestFile(problem.getID(), language.getID(), code);
		this.testOut = File.createTempFile(Lang.getString("Tester.3") + problem.getID(), ".out", new File(".")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		this.testErr = File.createTempFile(Lang.getString("Tester.6") + problem.getID(), ".err", new File(".")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
//		String delphiCompilerType;
//		if (Dcc32Utility.isDCC32(language)) {delphiCompilerType= "dcc32";} //$NON-NLS-1$ //$NON-NLS-2$
//		else {delphiCompilerType= "fpc";}
		
		//TESTCOMMAND= new File(new File(System.getProperty("user.dir")), "native").toString(); //$NON-NLS-1$ //$NON-NLS-2$
		boolean nativeFound=false;
		if (new File(new File(System.getProperty("user.dir")), "native").exists())
			{
			TESTCOMMAND=System.getProperty("user.dir")+File.separator+"native";
			nativeFound=true;
			}
		if (new File(new File(System.getProperty("user.dir")), "native.exe").exists())
			{
			TESTCOMMAND=System.getProperty("user.dir")+File.separator+"native.exe";
			nativeFound=true;
			}
		if (!nativeFound){
			JOptionPane.showMessageDialog(null, "Please rename the native component to 'native' or 'native.exe'");
			return;
		}	
			
		
		this.command = new String[] { TESTCOMMAND, language.getID(), Double.toString(timeLimit), fileToTest.getAbsolutePath(), testIn.getAbsolutePath(), testOut.getAbsolutePath(), testErr.getAbsolutePath() ,compiler.getName(),compiler.getPath(),compiler.getOption()};
		this.term = false;
		this.stderrOpen = true;
		this.stdoutOpen = true;
	}
	
	/*
	 * 1 File to be tested
	 * 2 temporary dir
	 * 3 compiler path
	 * 4 compiler options
	 * */

	/** Writes the submission to disk in order to compile and run it.
	 * @param problemID The id of the problem the submission is meant to solve.
	 * @param languageID The id of the programming language the submission is written in.
	 * @param code The submission itself. 
	 * @return Returns a File reference to the newly created file.
	 * @throws IOException
	 */
	private File createTestFile(String problemID, String languageID, String code) throws IOException {
		File testFile = File.createTempFile(Lang.getString("Tester.9") + problemID, "." + languageID, new File(".")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		FileWriter w = new FileWriter(testFile);
		try {
			w.write(code);
		} finally {
			w.close();
		}
		return testFile;
	}
	
	/** Registers a TestCompletedEventListener that will be called back once the <code>native</code> program has returned.
	 * @param listener The listener to register.
	 */
	public void addTestCompletedEventListener(TestCompletedEventListener listener) {
		testCompletedSignal.registerListener(listener);
	}
	
	/** The thread's run method.
	 * Executes <code>native</code> and waits for it to exit.
	 * Then extracts the judging suggestion and calls all registered <code>TestCompletedEventListener</code>s.
	 */
	public void run()
	{
		try {
			
			final StringBuffer p_stdout = new StringBuffer();
			final StringBuffer p_stderr = new StringBuffer();
			
			Diff diff = null; 
			
			try {
				//System.err.print("Executing: ");
				//for(int i=0; i<command.length; ++i) System.err.print(command[i] + " ");
				//System.err.println();
				testprocess = Runtime.getRuntime().exec(command);
				// WORKAROUND
				/*
				try {
					Thread.sleep(5);
				} catch(java.lang.InterruptedException e)
				{}*/
				
				int errvalue = testprocess.waitFor();
				if(errvalue == 2) {
					p_stderr.append(Lang.getString("Tester.12")); //$NON-NLS-1$
				} else {			
					char[] buffer = new char[1024];
					int len;
					FileReader reader = new FileReader(testErr);
					try {
						while((len = reader.read(buffer, 0, 1024)) != -1) {
							p_stderr.append(buffer, 0, len);
						}
					}
					finally {
						reader.close();
					}
				}
			} catch(IOException e) {
				throw new IOException(Lang.getString("Tester.13")); //$NON-NLS-1$
			}
		
			int i = p_stderr.lastIndexOf("\n", p_stderr.length()-2); //$NON-NLS-1$
			String suggestion = p_stderr.substring(i+1);
			
			testCompletedSignal.emitTestCompleted(new TestCompletedEvent(this, testOut, p_stderr.toString(), suggestion));
		} catch (IOException e) {
			testCompletedSignal.emitTestInterrupted(e.toString());
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			fileToTest.delete();
		}
	}

	/** <b>Not needed any more</b>
	 * @return
	 */
	private boolean inputsClosed() {
		return !(stderrOpen || stdoutOpen);
	}

	/** <b>Not needed any more</b>
	 * @return
	 */
	private synchronized boolean terminated() {
		return term; 
	}
	
	/** Delete all temporary files, i.e. the file to which stderr and stdout were written to.
	 */
	public void deleteAllTemporaries() {
		testErr.delete();
		testOut.delete();
	}
}
